@@ -134,9 +134,8 @@ end
 	
 	
 	for field in allFields:
-		fieldType = model.ResolveFieldType(field)
 		fieldName = GetPrivateName(field)
-		if fieldType is not null and not model.IsEnum(fieldType):
+		if model.IsNodeField(field):
 		
 %>			if (null != ${fieldName})
 			{
@@ -188,6 +187,10 @@ end
 			Output.WriteLine("""
 		[System.Xml.Serialization.XmlAttribute,
 		System.ComponentModel.DefaultValue(${field.Type}.None)]""")
+		elif model.IsCollectionField(field):
+			Output.WriteLine("""
+		[System.Xml.Serialization.XmlArray]
+		[System.Xml.Serialization.XmlArrayItem(typeof(${model.GetCollectionItemType(model.ResolveFieldType(field))}))]""")
 		else:
 			Output.WriteLine("""
 		[System.Xml.Serialization.XmlElement]""")
