@@ -27,10 +27,9 @@
 #endregion
 
 using System.Reflection;
+using System.Security;
 using System.Security.Permissions;
 
-[assembly: ReflectionPermission(SecurityAction.RequestMinimum,
-								ReflectionEmit=true)]
 [assembly: AssemblyTitle("boo - an extensible programming language for the CLI")]
 [assembly: AssemblyDescription("")]
 [assembly: AssemblyConfiguration("")]
@@ -39,6 +38,6 @@ using System.Security.Permissions;
 [assembly: AssemblyCopyright("(C) 2003-2007 Rodrigo Barreto de Oliveira")]
 [assembly: AssemblyTrademark("")]
 [assembly: AssemblyCulture("")]
-[assembly: AssemblyVersion("2.0.0.0")]
+[assembly: AssemblyVersion("2.0.9.2")]
 [assembly: AssemblyDelaySign(false)]
-
+[assembly: AllowPartiallyTrustedCallers]
