@@ -30,8 +30,11 @@ namespace Boo.Lang.Compiler.Ast
 {
 	using System;
 	using System.IO;
+	using System.Text;
 	using System.Xml.Serialization;
-	
+	using System.Collections.Generic;
+	using System.Text.RegularExpressions;
+
 	public class AstUtil
 	{
 		public static bool IsOverloadableOperator(BinaryOperatorType op)
@@ -54,6 +57,9 @@ namespace Boo.Lang.Compiler.Ast
 				case BinaryOperatorType.NotMember:
 				case BinaryOperatorType.BitwiseOr:
 				case BinaryOperatorType.BitwiseAnd:
+				case BinaryOperatorType.ExclusiveOr:
+				case BinaryOperatorType.ShiftLeft:
+				case BinaryOperatorType.ShiftRight:
 				{
 					return true;
 				}
@@ -115,16 +121,88 @@ namespace Boo.Lang.Compiler.Ast
 			return false;
 		}
 
-		public static bool IsAssignment(Expression node)
+		public static BinaryOperatorKind GetBinaryOperatorKind(BinaryExpression expression)
 		{
-			if (node.NodeType == NodeType.BinaryExpression)
-			{
-				BinaryOperatorType binaryOperator = ((BinaryExpression)node).Operator;
-				return IsAssignmentOperator(binaryOperator);
-			}
-			return false;
+			return GetBinaryOperatorKind(expression.Operator, false);
 		}
 
+		public static BinaryOperatorKind GetBinaryOperatorKind(BinaryExpression expression, bool exact)
+		{
+			return GetBinaryOperatorKind(expression.Operator, exact);
+		}
+
+		public static BinaryOperatorKind GetBinaryOperatorKind(BinaryOperatorType op)
+		{
+			return GetBinaryOperatorKind(op, false);
+		}
+
+		public static BinaryOperatorKind GetBinaryOperatorKind(BinaryOperatorType op, bool exact)
+		{
+			switch (op) {
+				case BinaryOperatorType.Addition:
+				case BinaryOperatorType.Subtraction:
+				case BinaryOperatorType.Multiply:
+				case BinaryOperatorType.Division:
+				case BinaryOperatorType.Modulus:
+				case BinaryOperatorType.Exponentiation:
+					return BinaryOperatorKind.Arithmetic;
+
+				case BinaryOperatorType.LessThan:
+				case BinaryOperatorType.LessThanOrEqual:
+				case BinaryOperatorType.GreaterThan:
+				case BinaryOperatorType.GreaterThanOrEqual:
+				case BinaryOperatorType.Equality:
+				case BinaryOperatorType.Inequality:
+				case BinaryOperatorType.Match:
+				case BinaryOperatorType.NotMatch:
+				case BinaryOperatorType.ReferenceEquality:
+				case BinaryOperatorType.ReferenceInequality:
+					return BinaryOperatorKind.Comparison;
+
+				case BinaryOperatorType.TypeTest:
+				case BinaryOperatorType.Member:
+				case BinaryOperatorType.NotMember:
+					return exact
+						? BinaryOperatorKind.TypeComparison
+						: BinaryOperatorKind.Comparison;
+
+				case BinaryOperatorType.Assign:
+					return BinaryOperatorKind.Assignment;
+
+				case BinaryOperatorType.InPlaceAddition:
+				case BinaryOperatorType.InPlaceSubtraction:
+				case BinaryOperatorType.InPlaceMultiply:
+				case BinaryOperatorType.InPlaceDivision:
+				case BinaryOperatorType.InPlaceModulus:
+				case BinaryOperatorType.InPlaceBitwiseAnd:
+				case BinaryOperatorType.InPlaceBitwiseOr:
+				case BinaryOperatorType.InPlaceExclusiveOr:
+				case BinaryOperatorType.InPlaceShiftLeft:
+				case BinaryOperatorType.InPlaceShiftRight:
+					return exact
+						? BinaryOperatorKind.InPlaceAssignment
+						: BinaryOperatorKind.Assignment;
+
+				case BinaryOperatorType.Or:
+				case BinaryOperatorType.And:
+					return BinaryOperatorKind.Logical;
+
+				case BinaryOperatorType.BitwiseOr:
+				case BinaryOperatorType.BitwiseAnd:
+				case BinaryOperatorType.ExclusiveOr:
+				case BinaryOperatorType.ShiftLeft:
+				case BinaryOperatorType.ShiftRight:
+					return BinaryOperatorKind.Bitwise;
+			}
+			throw new NotSupportedException(String.Format("unknown operator: {0}", op));
+		}
+
+		public static bool IsAssignment(Node node)
+		{
+			if (node.NodeType == NodeType.BinaryExpression)
+				return GetBinaryOperatorKind((BinaryExpression) node) == BinaryOperatorKind.Assignment;
+			return false;
+		}
 
 		public static ClassDefinition GetParentClass(Node node)
 		{
@@ -238,9 +316,7 @@ namespace Boo.Lang.Compiler.Ast
 			{
 				BinaryExpression be = (BinaryExpression)node.ParentNode;
 				if (node == be.Left)
-				{
-					return IsAssignmentOperator(be.Operator);
-				}
+					return IsAssignment(be);
 			}
 			return false;
 		}
@@ -259,20 +335,7 @@ namespace Boo.Lang.Compiler.Ast
 			}
 			return false;
 		}
-		
-		public static bool IsAssignmentOperator(BinaryOperatorType op)
-		{
-			return BinaryOperatorType.Assign == op ||
-					BinaryOperatorType.InPlaceAddition == op ||
-					BinaryOperatorType.InPlaceSubtraction == op ||
-					BinaryOperatorType.InPlaceMultiply == op ||
-					BinaryOperatorType.InPlaceDivision == op ||
-					BinaryOperatorType.InPlaceBitwiseAnd == op ||
-					BinaryOperatorType.InPlaceBitwiseOr == op ||
-					BinaryOperatorType.InPlaceShiftLeft == op ||
-					BinaryOperatorType.InPlaceShiftRight == op;
-		}
-		
+
 		public static Constructor CreateConstructor(Node lexicalInfoProvider, TypeMemberModifiers modifiers)
 		{
 			Constructor constructor = new Constructor(lexicalInfoProvider.LexicalInfo);
@@ -280,7 +343,18 @@ namespace Boo.Lang.Compiler.Ast
 			constructor.IsSynthetic = true;
 			return constructor;
 		}
-		
+
+		public static Constructor CreateDefaultConstructor(TypeDefinition type)
+		{
+			TypeMemberModifiers modifiers = TypeMemberModifiers.Public;
+			if (type is Module)
+				modifiers = TypeMemberModifiers.Private;
+			if (type.IsAbstract)
+				modifiers = TypeMemberModifiers.Protected;
+
+			return CreateConstructor(type, modifiers);
+		}
+
 		public static ReferenceExpression CreateReferenceExpression(LexicalInfo li, string fullname)
 		{
 			ReferenceExpression e = CreateReferenceExpression(fullname);
@@ -320,7 +394,13 @@ namespace Boo.Lang.Compiler.Ast
 			UnaryExpression e = node as UnaryExpression;
 			return null == e ? false : e.Operator == UnaryOperatorType.Explode;
 		}
-		
+
+		public static bool IsIndirection(Node node)
+		{
+			UnaryExpression e = node as UnaryExpression;
+			return null == e ? false : e.Operator == UnaryOperatorType.Indirection;
+		}
+
 		private AstUtil()
 		{
 		}
@@ -340,7 +420,7 @@ namespace Boo.Lang.Compiler.Ast
 		
 		public static void DebugNode(Node node)
 		{
-			System.Console.WriteLine("{0}: {1} - {2}",
+			Console.WriteLine("{0}: {1} - {2}",
 					node.LexicalInfo,
 					node.NodeType,
 					SafeToCodeString(node));
@@ -357,5 +437,160 @@ namespace Boo.Lang.Compiler.Ast
 				return "<unavailable>";
 			}
 		}
+
+		//use this to build a type member name unique in the inheritance hierarchy.
+		public static string BuildUniqueTypeMemberName(TypeDefinition type, string name)
+		{
+			if (String.IsNullOrEmpty(name))
+				throw new ArgumentNullException("name");
+
+			StringBuilder nameBuilder = new StringBuilder("$");
+			nameBuilder.Append(name);
+			nameBuilder.Append("__");
+			nameBuilder.Append(type.QualifiedName);
+			if (type.HasGenericParameters)
+			{
+				nameBuilder.Append("_");
+				string[] parameterNames = Array.ConvertAll<GenericParameterDeclaration, string>(
+					type.GenericParameters.ToArray(),
+					delegate(GenericParameterDeclaration gpd) { return gpd.Name; });
+				foreach (string parameterName in parameterNames)
+				{
+					nameBuilder.Append("_");
+					nameBuilder.Append(parameterName);
+				}
+			}
+			nameBuilder.Replace('.', '_');
+			nameBuilder.Append("$");
+			return nameBuilder.ToString();
+		}
+
+		internal static Local GetLocalByName(Method method, string name)
+		{
+			if (method.Locals.Count == 0)
+				return null;
+
+			foreach (Local local in method.Locals) {
+				if (!local.PrivateScope && local.Name == name)
+					return local;
+			}
+			return null;
+		}
+
+		public static LexicalInfo SafeLexicalInfo(Node node)
+		{
+			if (null == node) return LexicalInfo.Empty;
+			LexicalInfo info = node.LexicalInfo;
+			if (info.IsValid) return info;
+			return SafeLexicalInfo(node.ParentNode);
+		}
+
+		public static string SafePositionOnlyLexicalInfo(Node node)
+		{
+			LexicalInfo info = SafeLexicalInfo(node);
+			return String.Format("({0},{1})", info.Line, info.Column);
+		}
+
+		public static ICollection<TValue> GetValues<TNode, TValue>(NodeCollection<TNode> nodes)
+			where TNode : LiteralExpression
+		{
+			List<TValue> values = new List<TValue>(nodes.Count);
+
+			foreach (TNode node in nodes)
+				values.Add((TValue) Convert.ChangeType(node.ValueObject, typeof(TValue)));
+
+			return values;
+		}
+
+		internal static bool AllCodePathsReturnOrRaise(Block block)
+		{
+			if (null == block || block.IsEmpty)
+				return false;
+
+			Node node = block.LastStatement;
+			NodeType last = node.NodeType;
+			switch (last)
+			{
+				case NodeType.ReturnStatement:
+				case NodeType.RaiseStatement:
+					return true;
+
+				case NodeType.Block:
+					return AllCodePathsReturnOrRaise((Block)node);
+
+				case NodeType.IfStatement:
+					IfStatement ifstmt = (IfStatement) node;
+					return
+						AllCodePathsReturnOrRaise(ifstmt.TrueBlock)
+						&& AllCodePathsReturnOrRaise(ifstmt.FalseBlock);
+
+				case NodeType.TryStatement:
+					TryStatement ts = (TryStatement) node;
+					if (!AllCodePathsReturnOrRaise(ts.ProtectedBlock))
+						return false;
+					//if (null != ts.FailureBlock && !EndsWithReturnStatement(ts.FailureBlock))
+					//	return false;
+					foreach (ExceptionHandler handler in ts.ExceptionHandlers)
+					{
+						if (!AllCodePathsReturnOrRaise(handler.Block))
+							return false;
+					}
+					return true;
+			}
+			return false;
+		}
+
+		internal static RegexOptions GetRegexOptions(RELiteralExpression re)
+		{
+			RegexOptions ro = RegexOptions.None;
+
+			if (string.IsNullOrEmpty(re.Options))
+				return ro;
+
+			foreach (char opt in re.Options)
+			{
+				switch (opt)
+				{
+					/* common flags */
+					case 'g':
+						break; //no-op on .NET, global by default
+					case 'i':
+						ro |= RegexOptions.IgnoreCase;
+						break;
+					case 'm':
+						ro |= RegexOptions.Multiline;
+						break;
+
+					/* perl|python flags */
+					case 's':
+						ro |= RegexOptions.Singleline;
+						break;
+					case 'x':
+						//TODO: parser support, no-op for now
+						//ro |= RegexOptions.IgnorePatternWhitespace;
+						break;
+					case 'l': //no-op on .NET, (l)ocale-aware by default
+						break;
+
+					/* boo-specific flags */
+					case 'n': //culture-(n)eutral
+						ro |= RegexOptions.CultureInvariant;
+						break;
+					case 'c':
+						ro |= RegexOptions.Compiled;
+						break;
+					case 'e':
+						ro |= RegexOptions.ExplicitCapture;
+						break;
+
+					default:
+						CompilerContext.Current.Errors.Add(
+							CompilerErrorFactory.InvalidRegexOption(re, opt));
+						break;
+				}
+			}
+			return ro;
+		}
 	}
 }
+
