@@ -45,15 +45,40 @@ namespace Boo.Lang.Compiler.Ast
 			if (null != _statements) _statements = null;
 		}
 		
+		public bool IsEmpty
+		{
+			get { return (null == _statements || Statements.IsEmpty); }
+		}
+
+		[Obsolete("HasStatements is Obsolete, use IsEmpty instead")]
 		public bool HasStatements
 		{
-			get
-			{
-				if (_statements == null) return false;
-				return _statements.Count > 0;
-			}
+			get { return !IsEmpty; }
 		}
 
+		public bool StartsWith<T>() where T : Statement
+		{
+			return !IsEmpty && (Statements.StartsWith<T>()
+			       || (FirstStatement is Block && ((Block) FirstStatement).StartsWith<T>()));
+		}
+
+		public bool EndsWith<T>() where T : Statement
+		{
+			return !IsEmpty && (Statements.EndsWith<T>()
+			       || (LastStatement is Block && ((Block) LastStatement).EndsWith<T>()));
+		}
+
+		public Statement FirstStatement
+		{
+			get { return IsEmpty ? null : Statements.First; }
+		}
+
+		public Statement LastStatement
+		{
+			get { return IsEmpty ? null : Statements.Last; }
+		}
+
+
 		public override Block ToBlock()
 		{
 			return this;
@@ -61,11 +86,20 @@ namespace Boo.Lang.Compiler.Ast
 		
 		public void Add(Statement stmt)
 		{
-			this.Statements.Add(stmt);
+			Block block = stmt as Block;
+			if (null != block)
+				Add(block);
+			else
+				this.Statements.Add(stmt);
 		}
 		
 		public void Add(Block block)
 		{
+			if (block.HasAnnotations)
+			{
+				this.Statements.Add(block);
+				return;
+			}
 			this.Statements.Extend(block.Statements);
 		}
 		
@@ -83,5 +117,14 @@ namespace Boo.Lang.Compiler.Ast
 		{
 			this.Statements.Insert(index, stmt);
 		}
+
+		public Statement Simplify()
+		{
+			if (IsEmpty)
+				return this;
+			if (Statements.Count > 1 || HasAnnotations)
+				return this;
+			return Statements[0];
+		}
 	}
 }
