@@ -48,7 +48,7 @@ namespace Boo.Lang.Compiler.Ast
 		
 		public static bool Matches(Block lhs, Block rhs)
 		{
-			return lhs == null ? rhs == null || !rhs.HasStatements : lhs.Matches(rhs);
+			return lhs == null ? rhs == null || rhs.IsEmpty : lhs.Matches(rhs);
 		}
 
 		public static bool AllMatch<T>(IEnumerable<T> lhs, IEnumerable<T> rhs) where T : Node
@@ -154,6 +154,13 @@ namespace Boo.Lang.Compiler.Ast
 				_annotations[key] = value;
 			}
 		}
+
+
+
+		public bool HasAnnotations
+		{
+			get { return _annotations != null && _annotations.Count > 0; }
+		}
 		
 		public void Annotate(object key)
 		{
@@ -258,7 +265,7 @@ namespace Boo.Lang.Compiler.Ast
 			return false;
 		}
 		
-		private class ReplaceVisitor : DepthFirstTransformer
+		private sealed class ReplaceVisitor : DepthFirstTransformer
 		{
 			NodePredicate _predicate;
 			Node _template;	
@@ -349,20 +356,84 @@ namespace Boo.Lang.Compiler.Ast
 			new Visitors.BooPrinterVisitor(writer).Visit(this);
 			return writer.ToString();
 		}
-		
+
+
+		///<summary>
+		///Returns the closest ancestor node of type <paramref name="ancestorType"/>
+		///or null if no ancestor of requested type has been found.
+		///</summary>
+		///<param name="ancestorType">The type of node you request.</param>
 		public Node GetAncestor(NodeType ancestorType)
+		{
+			return GetAncestor(ancestorType, int.MaxValue);
+		}
+
+		///<summary>
+		///Returns the closest ancestor node of type <paramref name="ancestorType"/>
+		///within <paramref name="limitDepth"/> or null if no ancestor of requested
+		///type has been found.
+		///</summary>
+		///<param name="ancestorType">The type of node you request.</param>
+		///<param name="limitDepth">Maximum depth difference from this node to ancestor.</param>
+		public Node GetAncestor(NodeType ancestorType, int limitDepth)
 		{
 			Node parent = this.ParentNode;
-			while (null != parent)
+			while (null != parent && limitDepth > 0)
 			{
 				if (ancestorType == parent.NodeType)
-				{
 					return parent;
-				}
+				parent = parent.ParentNode;
+				limitDepth--;
+			}
+			return null;
+		}
+
+		///<summary>
+		///Returns the closest ancestor node of type <paramref name="TAncestor"/>
+		///or null if no ancestor of requested type has been found.
+		///</summary>
+		///<param name="TAncestor">The type of node you request.</param>
+		public TAncestor GetAncestor<TAncestor>() where TAncestor : Node
+		{
+			Node parent = this.ParentNode;
+			while (parent != null)
+			{
+				TAncestor ancestor = parent as TAncestor;
+				if (null != ancestor)
+					return ancestor;
 				parent = parent.ParentNode;
 			}
 			return null;
 		}
-	}
 
+		///<summary>
+		///Returns the farthest ancestor node of type <paramref name="TAncestor"/>
+		///or null if no ancestor of requested type has been found.
+		///</summary>
+		///<param name="TAncestor">The type of node you request.</param>
+		public TAncestor GetRootAncestor<TAncestor>() where TAncestor : Node
+		{
+			TAncestor root = null;
+			foreach (TAncestor ancestor in GetAncestors<TAncestor>())
+				root = ancestor;
+			return root;
+		}
+
+		///<summary>
+		///Yields <paramref name="TAncestor"/> ancestors in order from closest to farthest from this node.
+		///</summary>
+		///<param name="TAncestor">The type of node you request.</param>
+		public IEnumerable<TAncestor> GetAncestors<TAncestor>() where TAncestor : Node
+		{
+			Node parent = this.ParentNode;
+			while (parent != null)
+			{
+				TAncestor ancestor = parent as TAncestor;
+				if (null != ancestor)
+					yield return ancestor;
+				parent = parent.ParentNode;
+			}
+		}
+	}
 }
+
