@@ -37,22 +37,22 @@ namespace Boo.Lang.Compiler.Ast
 	/// <summary>
 	/// Node collection base class.
 	/// </summary>
-	public class NodeCollection<T> : ICollection, ICloneable, IEnumerable<T>
+	public class NodeCollection<T> : ICollection<T>, ICollection, ICloneable, IEquatable<NodeCollection<T>>
 		where T : Node
 	{
 		protected Node _parent;
 
-		protected List _list;
+		protected List<T> _list;
 
 		protected NodeCollection()
 		{
-			_list = new List();
+			_list = new List<T>();
 		}
 
 		protected NodeCollection(Node parent)
 		{
 			_parent = parent;
-			_list = new List();
+			_list = new List<T>();
 		}
 
 		protected NodeCollection(Node parent, IEnumerable<T> list)
@@ -60,18 +60,28 @@ namespace Boo.Lang.Compiler.Ast
 			if (null == list) throw new ArgumentNullException("list");
 
 			_parent = parent;
-			_list = new List(list);
+			_list = new List<T>(list);
 		}
 
 		public T this[int index]
 		{
-			get { return (T)_list[index]; }
+			get { return _list[index]; }
 			set { _list[index] = value; }
 		}
 
 		public IEnumerator<T> GetEnumerator()
 		{
-			foreach (T item in _list) yield return item;
+			return _list.GetEnumerator();
+		}
+
+		public Node ParentNode
+		{
+			get { return _parent; }
+		}
+
+		public bool IsEmpty
+		{
+			get { return (null == _list) || (0 == _list.Count); }
 		}
 
 		public int Count
@@ -79,6 +89,16 @@ namespace Boo.Lang.Compiler.Ast
 			get { return _list.Count; }
 		}
 
+		public int IndexOf(T item)
+		{
+			return _list.IndexOf(item);
+		}
+
+		public bool IsReadOnly
+		{
+			get { return false; }
+		}
+
 		public bool IsSynchronized
 		{
 			get { return false; }
@@ -90,6 +110,11 @@ namespace Boo.Lang.Compiler.Ast
 		}
 
 		public void CopyTo(Array array, int index)
+		{
+			((ICollection)_list).CopyTo(array, index);
+		}
+
+		public void CopyTo(T[] array, int index)
 		{
 			_list.CopyTo(array, index);
 		}
@@ -106,7 +131,7 @@ namespace Boo.Lang.Compiler.Ast
 
 		public T[] ToArray()
 		{
-			return (T[])_list.ToArray(new T[_list.Count]);
+			return _list.ToArray();
 		}
 
 		public T[] ToReverseArray()
@@ -116,17 +141,39 @@ namespace Boo.Lang.Compiler.Ast
 			return array;
 		}
 
-		public T[] Select(NodeType type)
+		public IEnumerable<TNode> OfType<TNode>() where TNode : Node
 		{
-			List result = new List();
 			foreach (Node node in _list)
 			{
-				if (node.NodeType == type)
-				{
-					result.Add(node);
-				}
+				TNode match = node as TNode;
+				if (null != match)
+					yield return match;
 			}
-			return (T[])result.ToArray(new T[result.Count]);
+		}
+
+		public IEnumerable<T> Except<UnwantedNodeType>() where UnwantedNodeType : T
+		{
+			foreach (T node in _list)
+				if (!(node is UnwantedNodeType))
+					yield return node;
+		}
+
+		public IEnumerable<T> Except<UnwantedNodeType,UnwantedNodeType2>()
+			where UnwantedNodeType : T
+			where UnwantedNodeType2 : T
+		{
+			foreach (T node in _list)
+				if (!(node is UnwantedNodeType) && !(node is UnwantedNodeType2))
+					yield return node;
+		}
+
+		public T[] Select(NodeType type)
+		{
+			List<T> result = new List<T>();
+			foreach (T node in _list)
+				if (node.NodeType == type)
+					result.Add(node);
+			return result.ToArray();
 		}
 
 		protected IEnumerable InternalPopRange(int begin)
@@ -134,19 +181,29 @@ namespace Boo.Lang.Compiler.Ast
 			return _list.PopRange(begin);
 		}
 
-		public bool ContainsNode(T node)
+		public bool Contains(T node)
 		{
-			return _list.ContainsReference(node);
+			foreach (T n in _list)
+			{
+				if (n == node) return true;
+			}
+			return false;
 		}
 
-		public bool Contains(Predicate condition)
+		[Obsolete("Use Contains(T node) instead.")]
+		public bool ContainsNode(T node)
+		{
+			return Contains(node);
+		}
+
+		public bool Contains(System.Predicate<T> condition)
 		{
 			return _list.Contains(condition);
 		}
 
 		public bool ContainsEntity(Boo.Lang.Compiler.TypeSystem.IEntity entity)
 		{
-			foreach (Node node in _list)
+			foreach (T node in _list)
 			{
 				if (entity == node.Entity)
 				{
@@ -174,26 +231,26 @@ namespace Boo.Lang.Compiler.Ast
 			return null;
 		}
 
-		public object Clone()
+		public virtual object Clone()
 		{
 			NodeCollection<T> clone = (NodeCollection<T>)Activator.CreateInstance(GetType());
-			List cloneList = clone._list;
-			foreach (Node node in _list)
+			List<T> cloneList = clone._list;
+			foreach (T node in _list)
 			{
-				cloneList.Add(node.Clone());
+				cloneList.Add((T)node.CloneNode());
 			}
 			return clone;
 		}
 
 		public void ClearTypeSystemBindings()
 		{
-			foreach (Node node in _list)
+			foreach (T node in _list)
 			{
 				node.ClearTypeSystemBindings();
 			}
 		}
 
-		protected List InnerList
+		protected List<T> InnerList
 		{
 			get
 			{
@@ -204,13 +261,13 @@ namespace Boo.Lang.Compiler.Ast
 		internal void InitializeParent(Node parent)
 		{
 			_parent = parent;
-			foreach (Node node in _list)
+			foreach (T node in _list)
 			{
 				node.InitializeParent(_parent);
 			}
 		}
 
-		public void Reject(Predicate condition)
+		public void Reject(System.Predicate<T> condition)
 		{
 			if (null == condition)
 			{
@@ -218,7 +275,7 @@ namespace Boo.Lang.Compiler.Ast
 			}
 
 			int index = 0;
-			foreach (Node node in ToArray())
+			foreach (T node in ToArray())
 			{
 				if (condition(node))
 				{
@@ -242,7 +299,7 @@ namespace Boo.Lang.Compiler.Ast
 		{
 			foreach (T item in items)
 			{
-				InnerList.Add(item.CloneNode());
+				Add((T)item.CloneNode());
 			}
 		}
 
@@ -296,21 +353,29 @@ namespace Boo.Lang.Compiler.Ast
 			InnerList.Insert(index, item);
 		}
 
-		public void Remove(Node item)
+		public bool Remove(T item)
 		{
-			InnerList.Remove(item);
+			return ((ICollection<T>)_list).Remove(item);
 		}
 
 		override public int GetHashCode()
 		{
-			return base.GetHashCode();
+			return _list.GetHashCode();
 		}
 
-		override public bool Equals(object rhs)
+		override public bool Equals(object other)
 		{
-			NodeCollection<T> other = rhs as NodeCollection<T>;
 			if (null == other) return false;
+			if (this == other) return true;
 
+			NodeCollection<T> collection = other as NodeCollection<T>;
+			return Equals(collection);
+		}
+
+		public bool Equals(NodeCollection<T> other)
+		{
+			if (null == other) return false;
+			if (this == other) return true;
 			if (InnerList.Count != other.Count) return false;
 
 			IEnumerator<T> enumerator = other.GetEnumerator();
@@ -340,5 +405,69 @@ namespace Boo.Lang.Compiler.Ast
 			}
 		}
 
+		public NodeCollection<TNew> Cast<TNew>() where TNew : Node
+		{
+			return Cast<TNew>(0);
+		}
+
+		public NodeCollection<TNew> Cast<TNew>(int index) where TNew : Node
+		{
+			if (index < 0 || index > Count)
+				throw new ArgumentOutOfRangeException("index");
+
+			if (0 == (Count - index)) //empty or no item to cast
+				return Empty<TNew>(ParentNode);
+
+			NodeCollection<TNew> nodes = new NodeCollection<TNew>(ParentNode);
+			int i = -1;
+
+			foreach (T node in _list)
+			{
+				++i;
+				if (i < index)
+					continue;
+
+				TNew cnode = node as TNew;
+				if (null == cnode)
+				{
+					ExpressionStatement es = node as ExpressionStatement;
+					if (null != es)
+						cnode = es.Expression as TNew;
+				}
+				if (null == cnode)
+					throw new InvalidCastException(
+						string.Format("Cannot cast item #{0} from `{1}` to `{2}`",
+						              i+1, node.GetType(), typeof(TNew)));
+
+				nodes.Add(cnode);
+			}
+			return nodes;
+		}
+
+		public static NodeCollection<TNode> Empty<TNode>(Node parent) where TNode : Node
+		{
+			//TODO: cache?
+			return new NodeCollection<TNode>(parent);
+		}
+
+		public bool StartsWith<TNode>() where TNode : T
+		{
+			return !IsEmpty && (First is TNode);
+		}
+
+		public bool EndsWith<TNode>() where TNode : T
+		{
+			return !IsEmpty && (Last is TNode);
+		}
+
+		public T First
+		{
+			get { return (IsEmpty ? null : _list[0]); }
+		}
+
+		public T Last
+		{
+			get { return (IsEmpty ? null : _list[Count - 1]); }
+		}
 	}
 }
