@@ -49,5 +49,10 @@ namespace Boo.Lang.Compiler.Ast
 		{
 			this.Value = value;
 		}
+
+		public override object ValueObject
+		{
+			get { return this.Value; }
+		}
 	}
 }
