@@ -31,6 +31,7 @@ using System.Text.RegularExpressions;
 using System.Globalization;
 using System.IO;
 using Boo.Lang.Compiler.Ast;
+using System.Collections.Generic;
 
 namespace Boo.Lang.Compiler.Ast.Visitors
 {
@@ -179,7 +180,7 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 		
 		private void WriteBlockStatements(Block b)
 		{
-			if (0 == b.Statements.Count)
+			if (b.IsEmpty)
 			{
 				WritePass();
 			}
@@ -506,8 +507,50 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 		override public void OnGenericParameterDeclaration(GenericParameterDeclaration gp)
 		{
 			Write(gp.Name);
+			if (gp.BaseTypes.Count > 0 || gp.Constraints != GenericParameterConstraints.None)
+			{
+				Write("(");
+				
+				WriteCommaSeparatedList(gp.BaseTypes);
+				
+				if (gp.Constraints != GenericParameterConstraints.None)
+				{
+					if (gp.BaseTypes.Count != 0)
+					{
+						Write(", ");
+					}
+					WriteGenericParameterConstraints(gp.Constraints);
+				}
+
+				Write(")");
+			}
 		}
-		
+
+		private void WriteGenericParameterConstraints(GenericParameterConstraints constraints)
+		{
+			List<string> constraintStrings = new List<string>();
+
+			if ((constraints & GenericParameterConstraints.ReferenceType) != GenericParameterConstraints.None)
+			{
+				constraintStrings.Add("class");
+			}
+			if ((constraints & GenericParameterConstraints.ValueType) != GenericParameterConstraints.None)
+			{
+				constraintStrings.Add("struct");
+			}
+			if ((constraints & GenericParameterConstraints.Constructable) != GenericParameterConstraints.None)
+			{
+				constraintStrings.Add("constructor");
+			}
+
+			Write(string.Join(", ", constraintStrings.ToArray()));
+		}
+
+		private KeyValuePair<T, string> CreateTranslation<T>(T value, string translation)
+		{
+			return new KeyValuePair<T, string>(value, translation);
+		}
+
 		override public void OnTypeofExpression(TypeofExpression node)
 		{
 			Write("typeof(");
@@ -869,7 +912,10 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 
 		override public void OnStringLiteralExpression(StringLiteralExpression e)
 		{
-			WriteStringLiteral(e.Value);
+			if (e != null && e.Value != null)
+				WriteStringLiteral(e.Value);
+			else
+				WriteKeyword("null");
 		}
 		
 		override public void OnCharLiteralExpression(CharLiteralExpression e)
@@ -959,10 +1005,10 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 			WriteIndented(node.Name);
 			Write(" ");
 			WriteCommaSeparatedList(node.Arguments);
-			if (node.Block.Statements.Count > 0)
+			if (!node.Body.IsEmpty)
 			{
 				WriteLine(":");
-				WriteBlock(node.Block);
+				WriteBlock(node.Body);
 			}
 			else
 			{
@@ -1257,6 +1303,16 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 				{
 					return "~";
 				}
+
+				case UnaryOperatorType.AddressOf:
+				{
+					return "&";
+				}
+
+				case UnaryOperatorType.Indirection:
+				{
+					return "*";
+				}
 			}
 			throw new ArgumentException("op");
 		}
@@ -1275,6 +1331,11 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 					return "=~";
 				}
 				
+				case BinaryOperatorType.NotMatch:
+				{
+					return "!~";
+				}
+
 				case BinaryOperatorType.Equality:
 				{
 					return "==";
@@ -1320,6 +1381,11 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 					return "*=";
 				}
 				
+				case BinaryOperatorType.InPlaceModulus:
+				{
+					return "%=";
+				}
+
 				case BinaryOperatorType.InPlaceExclusiveOr:
 				{
 					return "^=";
@@ -1603,7 +1669,6 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 		void WriteParameterList(ParameterDeclarationCollection items, string st, string ed)
 		{
 			Write(st);
-			int last = items.Count-1;
 			int i = 0;
 			foreach (ParameterDeclaration item in items)
 			{
@@ -1611,7 +1676,7 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 				{
 					Write(", ");
 				}
-				if (i == last && items.VariableNumber)
+				if (item.IsParamArray)
 				{
 					Write("*");
 				}
