@@ -31,7 +31,8 @@ namespace Boo.Lang.Compiler.Pipelines
 	using System;
 	using System.IO;
 	using System.Reflection;
-	
+	using Boo.Lang.Compiler.Util;
+
 	public class Parse : CompilerPipeline
 	{
 		static Type _defaultParserStepType;
@@ -44,15 +45,18 @@ namespace Boo.Lang.Compiler.Pipelines
 			}
 			return (ICompilerStep)Activator.CreateInstance(_defaultParserStepType);
 		}
-		
+
 		static Assembly FindParserAssembly()
 		{
 			Assembly thisAssembly = typeof(Parse).Assembly;
-			string thisLocation = thisAssembly.Location;
-			string parserLocation = thisLocation.Substring(0, thisLocation.Length-"Boo.Lang.Compiler.dll".Length) + "Boo.Lang.Parser.dll";
+			string thisLocation = Permissions.HasDiscoveryPermission ?  thisAssembly.Location : "";
+			string parserLocation = thisLocation.EndsWith("Boo.Lang.Compiler.dll")
+                	? thisLocation.Substring(0, thisLocation.Length - "Boo.Lang.Compiler.dll".Length) + "Boo.Lang.Parser.dll"
+                	: "";
+
 			return File.Exists(parserLocation)
-				? Assembly.LoadFrom(parserLocation)
-				: Assembly.Load(thisAssembly.FullName.Replace("Boo.Lang.Compiler", "Boo.Lang.Parser"));
+					? Assembly.LoadFrom(parserLocation)
+					: Assembly.Load(thisAssembly.FullName.Replace("Boo.Lang.Compiler", "Boo.Lang.Parser"));
 		}
 		
 		public Parse()
