@@ -30,40 +30,27 @@ namespace Boo.Lang.Compiler.Pipelines
 {
 	using Boo.Lang.Compiler.Steps;
 	
-	public class ResolveExpressions : Parse 
+	public class ResolveExpressions : ExpandMacros 
 	{
 		public ResolveExpressions()
 		{
-			Add(new InitializeTypeSystemServices());
-			Add(new PreErrorChecking());
-
 			Add(new ExpandAstLiterals());
 			
-			Add(new MergePartialClasses());
-			
-			Add(new InitializeNameResolutionService());
-			Add(new IntroduceGlobalNamespaces());
-			Add(new TransformCallableDefinitions());
-
-			Add(new BindTypeDefinitions());
-			Add(new BindGenericParameters());
-			Add(new BindNamespaces());
-			Add(new BindBaseTypes());
-			Add(new BindAndApplyAttributes());
-			
-			Add(new ExpandMacros());
 			Add(new IntroduceModuleClasses());
 			Add(new NormalizeStatementModifiers());
 			Add(new NormalizeTypeAndMemberDefinitions());
-			
+			Add(new NormalizeOmittedExpressions());
+
 			Add(new BindTypeDefinitions());
 			Add(new BindGenericParameters());
 			Add(new BindEnumMembers());
 			Add(new BindBaseTypes());
+			Add(new CheckMemberTypes());
 
 			Add(new BindMethods());
 			Add(new ResolveTypeReferences());
 			Add(new BindTypeMembers());
+			Add(new CheckGenericConstraints());
 			
 			Add(new ProcessInheritedAbstractMembers());
 			Add(new CheckMemberNames());
