@@ -30,6 +30,7 @@ using System;
 using Boo.Lang.Compiler;
 using Boo.Lang.Compiler.Ast;
 using Boo.Lang.Compiler.TypeSystem;
+using Boo.Lang.Compiler.TypeSystem.Services;
 
 namespace Boo.Lang.Compiler.Steps
 {
@@ -210,7 +211,7 @@ namespace Boo.Lang.Compiler.Steps
 
 		protected virtual void EnsureRelatedNodeWasVisited(Node sourceNode, IEntity entity)
 		{
-			IInternalEntity internalEntity = entity as IInternalEntity;
+			IInternalEntity internalEntity = GetConstructedInternalEntity(entity);
 			if (null != internalEntity)
 			{
 				Node node = internalEntity.Node;
@@ -221,6 +222,19 @@ namespace Boo.Lang.Compiler.Steps
 			}
 		}
 
+		protected static IInternalEntity GetConstructedInternalEntity(IEntity entity)
+		{
+			IConstructedMethodInfo constructedMethod = entity as IConstructedMethodInfo;
+			if (null != constructedMethod)
+				entity = constructedMethod.GenericDefinition;
+
+			IConstructedTypeInfo constructedType = entity as IConstructedTypeInfo;
+			if (null != constructedType)
+				entity = constructedType.GenericDefinition;
+
+			return entity as IInternalEntity;
+		}
+
 		protected bool WasVisited(Node node)
 		{
 			return node.ContainsAnnotation(VisitedAnnotationKey);
