@@ -1,4 +1,4 @@
-﻿#region license
+#region license
 // Copyright (c) 2003, 2004, 2005 Rodrigo B. de Oliveira (rbo@acm.org)
 // All rights reserved.
 // 
@@ -28,33 +28,42 @@
 
 
 using System;
+using Boo.Lang.Compiler.TypeSystem.Internal;
 
 namespace Boo.Lang.Compiler.Steps
 {
 	using Boo.Lang.Compiler.Ast;
 	using Boo.Lang.Compiler.TypeSystem;
 	
+	/// <summary>
+	/// Pre-binds methods and constructors before resolving type references, 
+	/// to enable correct resolution of generic type references.
+	/// </summary>
 	public class BindMethods : AbstractVisitorCompilerStep
 	{
-		public BindMethods()
+		private InternalTypeSystemProvider _internalTypeSystemProvider;
+
+		public override void Initialize(CompilerContext context)
 		{
+			base.Initialize(context);
+			_internalTypeSystemProvider = My<InternalTypeSystemProvider>.Instance;
+
 		}
-		
 		override public void OnMethod(Method node)
 		{
-			if (null == node.Entity)
-			{
-				if (node.GenericParameters.Count == 0)
-				{
-					node.Entity = new InternalMethod(TypeSystemServices, node);
-				}
-				else
-				{
-					node.Entity = new InternalGenericMethod(TypeSystemServices, node);
-				}
-			}
+			EnsureEntityFor(node);
 			Visit(node.ExplicitInfo);
 		}
+
+		protected void EnsureEntityFor(TypeMember node)
+		{
+			_internalTypeSystemProvider.EntityFor(node);
+		}
+
+		public override void OnConstructor(Constructor node)
+		{
+			EnsureEntityFor(node);
+		}
 		
 		override public void OnExplicitMemberInfo(ExplicitMemberInfo node)
 		{
