@@ -31,6 +31,7 @@ using Boo.Lang;
 using Boo.Lang.Compiler;
 using Boo.Lang.Compiler.Ast;
 using Boo.Lang.Compiler.TypeSystem;
+using Boo.Lang.Compiler.TypeSystem.Builders;
 
 namespace Boo.Lang.Compiler.Steps
 {
@@ -43,6 +44,8 @@ namespace Boo.Lang.Compiler.Steps
 		Method _currentMethod;
 		
 		List _references;
+
+		List _recursiveReferences;
 		
 		Hash _referencedEntities;
 		
@@ -53,6 +56,7 @@ namespace Boo.Lang.Compiler.Steps
 		public ForeignReferenceCollector()
 		{
 			_references = new List();
+			_recursiveReferences = new List();
 			_referencedEntities = new Hash();
 		}
 		
@@ -183,10 +187,10 @@ namespace Boo.Lang.Compiler.Steps
 			// referenced entities turn into fields
 			foreach (ITypedEntity entity in Builtins.array(_referencedEntities.Keys))
 			{
-				Field field = builder.AddInternalField("__" + entity.Name + _context.AllocIndex(), entity.Type);
+				Field field = builder.AddInternalField(_context.GetUniqueName(entity.Name), entity.Type);
 				_referencedEntities[entity] = field.Entity;
 			}
-			
+
 			// single constructor taking all referenced entities
 			BooMethodBuilder constructor = builder.AddConstructor();
 			constructor.Modifiers = TypeMemberModifiers.Public;			
@@ -212,6 +216,15 @@ namespace Boo.Lang.Compiler.Steps
 							CodeBuilder.CreateReference(entity));
 				}
 			}
+
+			foreach (ReferenceExpression reference in _recursiveReferences)
+			{
+				reference.ParentNode.Replace(
+					reference,
+					CodeBuilder.MemberReferenceForEntity(
+						CodeBuilder.CreateSelfReference((IType)CurrentMethod.DeclaringType.Entity), 
+						CurrentMethod.Entity));
+			}
 		}
 		
 		public MethodInvocationExpression CreateConstructorInvocationWithReferencedEntities(IType type)
@@ -235,7 +248,19 @@ namespace Boo.Lang.Compiler.Steps
 				return CodeBuilder.CreateReference(entity);
 			}
 		}
-		
+
+		public override void OnMemberReferenceExpression(MemberReferenceExpression node)
+		{
+			if (IsRecursiveReference(node))
+			{
+				_recursiveReferences.Add(node);
+			}
+			else
+			{
+				Visit(node.Target);
+			}
+		}
+
 		override public void OnReferenceExpression(ReferenceExpression node)
 		{
 			if (IsForeignReference(node))
@@ -252,6 +277,11 @@ namespace Boo.Lang.Compiler.Steps
 			_references.Add(node);
 			_referencedEntities[entity] = null;
 		}
+
+		private bool IsRecursiveReference(Node node)
+		{
+			return (CurrentMethod != null && node.Entity == CurrentMethod.Entity);
+		}
 		
 		bool IsForeignReference(ReferenceExpression node)
 		{
