@@ -26,6 +26,9 @@
 // THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #endregion
 
+using Boo.Lang.Compiler.TypeSystem.Builders;
+using Boo.Lang.Compiler.TypeSystem.Reflection;
+using Boo.Lang.Compiler.Util;
 
 namespace Boo.Lang.Compiler.Steps
 {
@@ -95,7 +98,7 @@ namespace Boo.Lang.Compiler.Steps
 			_enumerator = _collector.CreateSkeletonClass("Enumerator",_generator.LexicalInfo);
 
 			// use a generic enumerator for the source type if possible
-			_sourceItemType = TypeSystemServices.GetGenericEnumerableItemType(_generator.Iterator.ExpressionType);			
+			_sourceItemType = TypeSystemServices.GetGenericEnumerableItemType(_generator.Iterator.ExpressionType);
 			if (_sourceItemType != null && _sourceItemType != TypeSystemServices.ObjectType)
 			{
 				_sourceEnumerableType = TypeSystemServices.IEnumerableGenericType.GenericInfo.ConstructType(_sourceItemType);
@@ -109,7 +112,7 @@ namespace Boo.Lang.Compiler.Steps
 			// Add base types
 			_enumerator.AddBaseType(_resultEnumeratorType);
 			_enumerator.AddBaseType(TypeSystemServices.Map(typeof(ICloneable)));
-			_enumerator.AddBaseType(TypeSystemServices.Map(typeof(IDisposable)));
+			_enumerator.AddBaseType(TypeSystemServices.IDisposableType);
 			
 			// Add fields
 			_enumeratorField = _enumerator.AddField("$$enumerator", _sourceEnumeratorType);
@@ -309,11 +312,11 @@ namespace Boo.Lang.Compiler.Steps
 		private void CreateDispose()
 		{
 			BooMethodBuilder dispose = _enumerator.AddVirtualMethod("Dispose", TypeSystemServices.VoidType);
-			if (TypeSystemServices.Map(typeof(IDisposable)).IsAssignableFrom(_sourceEnumeratorType))
-			{			
+			if (TypeSystemServices.IDisposableType.IsAssignableFrom(_sourceEnumeratorType))
+			{
 				dispose.Body.Add(CodeBuilder.CreateMethodInvocation(
 					CodeBuilder.CreateReference(_enumeratorField),
-					typeof(IDisposable).GetMethod("Dispose")));
+					Types.IDisposable.GetMethod("Dispose")));
 			}
 		}
 
@@ -329,27 +332,18 @@ namespace Boo.Lang.Compiler.Steps
 				if (entityType == EntityType.Property)
 				{
 					return TypeSystemServices.Map(
-						((ExternalType)type).ActualType.GetProperty(name));
+						external.ActualType.GetProperty(name));
 				}
-
 				else if (entityType == EntityType.Method)
 				{
 					return TypeSystemServices.Map(
-						((ExternalType)type).ActualType.GetMethod(name));
+						external.ActualType.GetMethod(name));
 
 				}
 			}
 
-			// For constructed types which aren't external we can use the GenericMapping to 
-			// (maybe) optimize things a little
-            if (type.ConstructedInfo != null)
-            {
-                return ((GenericConstructedType)type).GenericMapping.Map(
-                    GetMember(type.ConstructedInfo.GenericDefinition, name, entityType));
-            }
-
-            // For other cases we just scan through the members collection
-            return Array.Find<IEntity>(
+			// For other cases we just scan through the members collection
+            return Collections.FindFirst<IEntity>(
                 type.GetMembers(), 
                 delegate(IEntity e) { 
                     return entityType == e.EntityType && e.Name == name; 
