@@ -26,6 +26,8 @@
 // THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #endregion
 
+using Boo.Lang.Compiler.TypeSystem.Builders;
+using Boo.Lang.Compiler.TypeSystem.Internal;
 
 namespace Boo.Lang.Compiler.Steps
 {
@@ -221,28 +223,12 @@ namespace Boo.Lang.Compiler.Steps
 			methodBuilder.Method.LexicalInfo = generator.LexicalInfo;
 			_moveNext = methodBuilder.Entity;
 			
-			foreach (Local local in generator.Locals)
-			{
-				InternalLocal entity = (InternalLocal)local.Entity;
-				
-				Field field = _enumerator.AddInternalField("$" + entity.Name + "$" + _context.AllocIndex(), entity.Type);
-				_mapping[entity] = field.Entity;
-			}
-			generator.Locals.Clear();
-			
-			foreach (ParameterDeclaration parameter in generator.Parameters)
-			{
-				InternalParameter entity = (InternalParameter)parameter.Entity;
-				if (entity.IsUsed)
-				{
-					Field field = DeclareFieldInitializedFromConstructorParameter(_enumerator, _enumeratorConstructor,
-					                                                              entity.Name,
-					                                                              entity.Type);
-					_mapping[entity] = field.Entity;
-				}
-			}
+			TransformLocalsIntoFields(generator);
+
+			TransformParametersIntoFieldsInitializedByConstructor(generator);
 			
 			methodBuilder.Body.Add(CreateLabel(generator));
+
 			// Visit() needs to know the number of the finished state
 			_finishedStateNumber = _labels.Count;
 			LabelStatement finishedLabel = CreateLabel(generator);
@@ -273,7 +259,56 @@ namespace Boo.Lang.Compiler.Steps
 				methodBuilder.Body.Add(tryFailure);
 			}
 		}
-		
+
+		private void TransformParametersIntoFieldsInitializedByConstructor(Method generator)
+		{
+			foreach (ParameterDeclaration parameter in generator.Parameters)
+			{
+				InternalParameter entity = (InternalParameter)parameter.Entity;
+				if (entity.IsUsed)
+				{
+					Field field = DeclareFieldInitializedFromConstructorParameter(_enumerator, _enumeratorConstructor,
+					                                                              entity.Name,
+					                                                              entity.Type);
+					_mapping[entity] = field.Entity;
+				}
+			}
+		}
+
+		private void TransformLocalsIntoFields(Method generator)
+		{
+			foreach (Local local in generator.Locals)
+			{
+				InternalLocal entity = (InternalLocal)local.Entity;
+				if (IsExceptionHandlerVariable(entity))
+				{
+					AddToMoveNextMethod(local);
+					continue;
+				}
+
+				AddInternalFieldFor(entity);
+			}
+			generator.Locals.Clear();
+		}
+
+		private void AddToMoveNextMethod(Local local)
+		{
+			_moveNext.Method.Locals.Add(local);
+		}
+
+		private void AddInternalFieldFor(InternalLocal entity)
+		{
+			Field field = _enumerator.AddInternalField(Context.GetUniqueName(entity.Name), entity.Type);
+			_mapping[entity] = field.Entity;
+		}
+
+		private bool IsExceptionHandlerVariable(InternalLocal local)
+		{
+			Declaration originalDeclaration = local.OriginalDeclaration;
+			if (originalDeclaration == null) return false;
+			return originalDeclaration.ParentNode is ExceptionHandler;
+		}
+
 		MethodInvocationExpression CallMethodOnSelf(IMethod method)
 		{
 			return CodeBuilder.CreateMethodInvocation(
@@ -339,7 +374,7 @@ namespace Boo.Lang.Compiler.Steps
 		                                                      string parameterName,
 		                                                      IType parameterType)
 		{
-			Field field = type.AddInternalField("$" + parameterName + _context.AllocIndex(), parameterType);
+			Field field = type.AddInternalField(Context.GetUniqueName(parameterName), parameterType);
 			InitializeFieldFromConstructorParameter(constructor, field, parameterName, parameterType);
 			return field;
 		}
@@ -384,7 +419,7 @@ namespace Boo.Lang.Compiler.Steps
 			ReplaceCurrentNode(CodeBuilder.CreateReference(node.LexicalInfo, _externalEnumeratorSelf));
 		}
 		
-		class TryStatementInfo
+		sealed class TryStatementInfo
 		{
 			internal TryStatement _statement;
 			internal TryStatementInfo _parent;
