@@ -26,6 +26,8 @@
 // THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #endregion
 
+using Boo.Lang.Compiler.TypeSystem.Builders;
+
 namespace Boo.Lang.Compiler.Steps
 {
 	using System;
@@ -111,18 +113,18 @@ namespace Boo.Lang.Compiler.Steps
 		
 		override public void LeaveReturnStatement(ReturnStatement node)
 		{
-			if (HasReturnType(_current))
-			{
-				Expression expression = node.Expression;
-				if (null != expression)
-				{
-					Expression newExpression = Convert(_current.ReturnType, expression);
-					if (null != newExpression)
-					{
-						node.Expression = newExpression;
-					}
-				}
-			}
+			Expression expression = node.Expression;
+			if (null == expression)
+				return;
+
+			if (!HasReturnType(_current))
+				return;
+			
+			Expression newExpression = Convert(_current.ReturnType, expression);
+			if (null == newExpression)
+				return;
+
+			node.Expression = newExpression;
 		}
 		
 		override public void LeaveExpressionPair(ExpressionPair pair)
@@ -319,13 +321,11 @@ namespace Boo.Lang.Compiler.Steps
 		{
 			ClassDefinition adaptor = FindAdaptor(to, from);
 			if (null == adaptor)
-			{
 				adaptor = CreateAdaptor(to, from);
-			}
 			return adaptor;
 		}
 		
-		class AdaptorRecord
+		sealed class AdaptorRecord
 		{
 			public readonly ICallableType To;
 			public readonly ICallableType From;
@@ -427,7 +427,7 @@ namespace Boo.Lang.Compiler.Steps
 		
 		void ReplaceEndInvokeTargetByGetAsyncDelegate(MethodInvocationExpression node)
 		{
-			Expression asyncResult = node.Arguments[node.Arguments.Count-1];
+			Expression asyncResult = node.Arguments.Last;
 			MemberReferenceExpression endInvoke = (MemberReferenceExpression)node.Target;
 			IType callableType = ((IMember)endInvoke.Entity).DeclaringType;
 			
@@ -435,7 +435,6 @@ namespace Boo.Lang.Compiler.Steps
 									CodeBuilder.CreateMethodInvocation(
 										CodeBuilder.CreateCast(_asyncResultType, asyncResult.CloneNode()),
 										_asyncResultTypeAsyncDelegateGetter));
-										
 		}
 		
 		bool IsCallableType(IType type)
