@@ -26,6 +26,8 @@
 // THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #endregion
 
+using Boo.Lang.Compiler.TypeSystem.Internal;
+
 namespace Boo.Lang.Compiler.Steps
 {
 	using Boo.Lang.Compiler;
@@ -64,8 +66,25 @@ namespace Boo.Lang.Compiler.Steps
 		override public void Run()
 		{
 			Visit(CompileUnit.Modules);
+
+			DetectOutputType();
 		}
-		
+
+		private void DetectOutputType()
+		{
+			if (Parameters.OutputType != CompilerOutputType.Auto)
+				return;
+
+			Parameters.OutputType = HasEntryPoint()
+				? CompilerOutputType.ConsoleApplication
+				: CompilerOutputType.Library;
+		}
+
+		private bool HasEntryPoint()
+		{
+			return null != ContextAnnotations.GetEntryPoint(Context);
+		}
+
 		override public void Dispose()
 		{
 			_booModuleAttributeType = null;
@@ -104,7 +123,7 @@ namespace Boo.Lang.Compiler.Steps
 				++removed;
 			}
 			
-			if (node.Globals.Statements.Count > 0)
+			if (!node.Globals.IsEmpty)
 			{
 				Method method = new Method(node.Globals.LexicalInfo);
 				method.IsSynthetic = true;
@@ -119,10 +138,7 @@ namespace Boo.Lang.Compiler.Steps
 				entryPoint = method;
 			}
 			
-			if (null != entryPoint)
-			{
-				ContextAnnotations.SetEntryPoint(Context, entryPoint);
-			}
+			SetEntryPointIfNecessary(entryPoint);
 			
 			if (hasModuleClass || _forceModuleClass || (moduleClass.Members.Count > 0))
 			{
@@ -132,16 +148,29 @@ namespace Boo.Lang.Compiler.Steps
 					moduleClass.Attributes.Add(CreateBooModuleAttribute());
 					node.Members.Add(moduleClass);
 				}
-				
-				moduleClass.Members.Add(AstUtil.CreateConstructor(node, TypeMemberModifiers.Private));
+
+				if (!moduleClass.HasInstanceConstructor)
+					moduleClass.Members.Add(AstUtil.CreateDefaultConstructor(node));
+
 				moduleClass.Modifiers = TypeMemberModifiers.Public |
 										TypeMemberModifiers.Final |
 										TypeMemberModifiers.Transient;
 				
-				((ModuleEntity)node.Entity).InitializeModuleClass(moduleClass);
+				((InternalModule)node.Entity).InitializeModuleClass(moduleClass);
 			}
 		}
-		
+
+		private void SetEntryPointIfNecessary(Method entryPoint)
+		{
+			if (null == entryPoint)
+				return;
+
+			if (Parameters.OutputType == CompilerOutputType.Library)
+				return;
+
+			ContextAnnotations.SetEntryPoint(Context, entryPoint);
+		}
+
 		ClassDefinition FindModuleClass(Module node)
 		{
 			ClassDefinition found = null;
@@ -184,9 +213,10 @@ namespace Boo.Lang.Compiler.Steps
 			}
 			else
 			{
-				module.Name = name = string.Format("__Module{0}__", _context.AllocIndex());
+				module.Name = name = Context.GetUniqueName("Module");
 			}
 			return name;
 		}
 	}
 }
+
