@@ -26,6 +26,9 @@
 // THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #endregion
 
+using System;
+using Boo.Lang.Compiler.TypeSystem;
+
 namespace Boo.Lang.Compiler.Steps
 {
 	using Boo.Lang.Compiler.Ast;
@@ -46,7 +49,7 @@ namespace Boo.Lang.Compiler.Steps
 		{
 			if (!node.IsVisibilitySet)
 			{
-				node.Modifiers |= TypeMemberModifiers.Public;
+				node.Modifiers |= Context.Parameters.DefaultTypeVisibility;
 			}
 		}
 		
@@ -65,27 +68,94 @@ namespace Boo.Lang.Compiler.Steps
 			LeaveTypeDefinition(node);
 			if (!node.HasInstanceConstructor && !node.IsStatic)
 			{
-				node.Members.Add(AstUtil.CreateConstructor(node, TypeMemberModifiers.Public));
+				node.Members.Add(AstUtil.CreateDefaultConstructor(node));
 			}
 		}
-		
+
+		override public void LeaveStructDefinition(StructDefinition node)
+		{
+			LeaveTypeDefinition(node);
+		}
+
 		override public void LeaveField(Field node)
 		{
 			if (!node.IsVisibilitySet)
 			{
-				node.Modifiers |= TypeMemberModifiers.Protected;
+				node.Visibility = Context.Parameters.DefaultFieldVisibility;
+
+				//protected field in a sealed type == private,
+				//so let the compiler mark them private automatically in order to get 
+				//unused members warnings for free (and to make IL analysis tools happy as a bonus)
+				if (node.IsProtected && node.DeclaringType.IsFinal)
+				{
+					node.Visibility = TypeMemberModifiers.Private;
+				}
 			}
+
+			LeaveMember(node);
 		}
-		
+
 		override public void LeaveProperty(Property node)
 		{
-			if (!node.IsVisibilitySet && null == node.ExplicitInfo)
+			NormalizeDefaultItemProperty(node);
+			NormalizePropertyModifiers(node);
+			LeaveMember(node);
+		}
+
+		private void NormalizeDefaultItemProperty(Property node)
+		{
+			if (!IsDefaultItemProperty(node)) return;
+
+			node.Name = "Item";
+			TypeDefinition declaringType = node.DeclaringType;
+			if (declaringType != null) AddDefaultMemberAttribute(declaringType, node);
+		}
+
+		private void AddDefaultMemberAttribute(TypeDefinition type, Property node)
+		{
+			if (!ContainsDefaultMemberAttribute(type))
 			{
-				node.Modifiers |= TypeMemberModifiers.Public;
+				Ast.Attribute attribute = CodeBuilder.CreateAttribute(
+					DefaultMemberAttributeStringConstructor(), 
+					new StringLiteralExpression(node.Name));
+				attribute.LexicalInfo = node.LexicalInfo;
+				type.Attributes.Add(attribute);
 			}
+		}
+
+		private IConstructor DefaultMemberAttributeStringConstructor()
+		{
+			return TypeSystemServices.Map(Types.DefaultMemberAttribute.GetConstructor(new System.Type[] {Types.String }));
+		}
+
+		private static bool ContainsDefaultMemberAttribute(TypeDefinition t)
+		{
+			foreach (Attribute a in t.Attributes)
+			{
+				if (a.Name.IndexOf("DefaultMember") >= 0)
+				{
+					return true;
+				}
+			}
+			return false;
+		}
+
+		private static bool IsDefaultItemProperty(Property node)
+		{
+			return (node.Name == "Item" || node.Name == "self")
+				&& node.Parameters.Count > 0
+				&& !node.IsStatic;
+		}
+
+		private void NormalizePropertyModifiers(Property node)
+		{
 			if (IsInterface(node.DeclaringType))
 			{
-				node.Modifiers |= TypeMemberModifiers.Abstract;
+				node.Modifiers = TypeMemberModifiers.Public | TypeMemberModifiers.Abstract;
+			}
+			else if (!node.IsVisibilitySet && null == node.ExplicitInfo)
+			{
+				node.Modifiers |= Context.Parameters.DefaultPropertyVisibility;
 			}
 
 			if (null != node.Getter)
@@ -98,8 +168,6 @@ namespace Boo.Lang.Compiler.Steps
 				SetPropertyAccessorModifiers(node, node.Setter);
 				node.Setter.Name = "set_" + node.Name;
 			}
-			
-			LeaveMember(node);
 		}
 
 		void SetPropertyAccessorModifiers(Property property, Method accessor)
@@ -139,27 +207,27 @@ namespace Boo.Lang.Compiler.Steps
 		
 		override public void LeaveEvent(Event node)
 		{
-			if (!node.IsVisibilitySet)
-			{
-				node.Modifiers |= TypeMemberModifiers.Public;
-			}
 			if (IsInterface(node.DeclaringType))
 			{
-				node.Modifiers |= TypeMemberModifiers.Abstract;
+				node.Modifiers = TypeMemberModifiers.Public | TypeMemberModifiers.Abstract;
+			}
+			else if (!node.IsVisibilitySet)
+			{
+				node.Modifiers |= Context.Parameters.DefaultEventVisibility;
 			}
 			LeaveMember(node);
 		}
 		
 		override public void LeaveMethod(Method node)
 		{
-			if (!node.IsVisibilitySet && null == node.ExplicitInfo
-				&& !(node.ParentNode.NodeType == NodeType.Property))
-			{
-				node.Modifiers |= TypeMemberModifiers.Public;
-			}
 			if (IsInterface(node.DeclaringType))
 			{
-				node.Modifiers |= TypeMemberModifiers.Abstract;
+				node.Modifiers = TypeMemberModifiers.Public | TypeMemberModifiers.Abstract;
+			}
+			else if (!node.IsVisibilitySet && null == node.ExplicitInfo
+				&& !(node.ParentNode.NodeType == NodeType.Property))
+			{
+				node.Modifiers |= Context.Parameters.DefaultMethodVisibility;
 			}
 			if (node.Name != null && node.Name.StartsWith("op_"))
 			{
@@ -210,11 +278,12 @@ namespace Boo.Lang.Compiler.Steps
 		
 		override public void LeaveConstructor(Constructor node)
 		{
-			if (!node.IsVisibilitySet)
-			{
-				node.Modifiers |= TypeMemberModifiers.Public;
-			}
-		}
+			if (node.IsVisibilitySet) return;
 
+			if (!node.IsStatic)
+				node.Modifiers |= Context.Parameters.DefaultMethodVisibility;
+			else
+				node.Modifiers |= TypeMemberModifiers.Private;
+		}
 	}
 }
