@@ -27,6 +27,8 @@
 #endregion
 
 using System;
+using System.Collections.Generic;
+using Boo.Lang.Compiler.TypeSystem.Internal;
 
 namespace Boo.Lang.Compiler.Steps
 {
@@ -91,20 +93,24 @@ namespace Boo.Lang.Compiler.Steps
 			member.Visibility = TypeMemberModifiers.Private;
 		}
 
-		void CheckExplicitMemberValidity(IExplicitMember node)
+		void CheckExplicitMemberValidity(IExplicitMember member)
 		{
-			IMember explicitMember = (IMember)GetEntity((Node)node);
+			Node node = (Node) member;
+			IMember explicitMember = (IMember)GetEntity(node);
 			if (explicitMember.DeclaringType.IsClass)
 			{
-				IType targetInterface = GetType(node.ExplicitInfo.InterfaceType);
+				IType targetInterface = GetType(member.ExplicitInfo.InterfaceType);
 				if (!targetInterface.IsInterface)
 				{
-					Error(CompilerErrorFactory.InvalidInterfaceForInterfaceMember((Node)node, node.ExplicitInfo.InterfaceType.Name));
+					Error(CompilerErrorFactory.InvalidInterfaceForInterfaceMember(node, member.ExplicitInfo.InterfaceType.Name));
 				}
-				
-				if (!explicitMember.DeclaringType.IsSubclassOf(targetInterface))
+				else if (!explicitMember.DeclaringType.IsSubclassOf(targetInterface))
 				{
-					Error(CompilerErrorFactory.InterfaceImplForInvalidInterface((Node)node, targetInterface.Name, ((TypeMember)node).Name));
+					Error(CompilerErrorFactory.InterfaceImplForInvalidInterface(node, targetInterface.Name, ((TypeMember)node).Name));
+				}
+				else if (!DoesExplicitMemberExistInInterface(explicitMember, targetInterface))
+				{
+					Error(CompilerErrorFactory.NotAMemberOfExplicitInterface((TypeMember) node, targetInterface));
 				}
 			}
 			else
@@ -113,6 +119,30 @@ namespace Boo.Lang.Compiler.Steps
 			}
 		}
 
+		bool DoesExplicitMemberExistInInterface(IMember member, IType iface)
+		{
+			//check the explicit decl exists in the interface
+			ICollection<IEntity> candidates = new List<IEntity>();
+			if (!iface.Resolve(candidates, member.Name, EntityType.Any))
+				return false;
+
+			//it exists! check if has a valid signature (ie. not an overload)
+			if (!(member is IMethod))
+				return true; //if a property no overload possible, return type conflict detected elsewhere
+
+			bool match = false;
+			foreach (IEntity candidate in candidates)
+			{
+				IMethod method = candidate as IMethod;
+				if (null == method)
+					continue;
+				match |= CallableSignature.AreSameParameters(((IMethod)member).GetParameters(), method.GetParameters());
+				if (match) //we're ok! skip the rest
+					break;
+			}
+			return match;
+		}
+
 		override public void LeaveInterfaceDefinition(InterfaceDefinition node)
 		{
 			MarkVisited(node);
@@ -169,8 +199,9 @@ namespace Boo.Lang.Compiler.Steps
                             {
                                 //Events and their corresponding Delegate Fields can have the same name
                                 //In such cases, we want the Event...
-                                if (inheritedImpl is ExternalEvent && oddjob is ExternalField && 
-                                    ((ExternalField)oddjob).Type.IsSubclassOf(TypeSystemServices.MulticastDelegateType))
+                                ExternalField oddField = oddjob as ExternalField;
+                                if (inheritedImpl is ExternalEvent && null != oddField
+                                    && oddField.Type.IsSubclassOf(TypeSystemServices.MulticastDelegateType))
                                 {
                                     continue;
                                 }
@@ -182,13 +213,15 @@ namespace Boo.Lang.Compiler.Steps
 					if( null != inheritedImpl)
 					{
 						if(inheritedImpl == entity)
-						{
 							return false; //Evaluating yourself is a very bad habit.
-						}
+
+						if (inheritedImpl is IExternalEntity && entity is IExternalEntity)
+							return true; //both sides are external, no need to check further
+
 						switch( entity.EntityType)
 						{
 							case EntityType.Method:
-								return CheckInheritedMethodImpl(inheritedImpl as IMethod, entity as IMethod);								
+								return CheckInheritedMethodImpl(inheritedImpl as IMethod, entity as IMethod);
 							case EntityType.Event:
 								return CheckInheritedEventImpl(inheritedImpl as IEvent, entity as IEvent);
 							case EntityType.Property:
@@ -202,10 +235,14 @@ namespace Boo.Lang.Compiler.Steps
 
 		bool CheckInheritedMethodImpl(IMethod impl, IMethod baseMethod)
 		{
-			if(TypeSystemServices.CheckOverrideSignature(impl, baseMethod))
+			if (TypeSystemServices.CheckOverrideSignature(impl, baseMethod))
 			{
-				if(impl.ReturnType == baseMethod.ReturnType)
+				IType baseReturnType = TypeSystemServices.GetOverriddenSignature(baseMethod, impl).ReturnType;
+				if (impl.ReturnType == baseReturnType)
+				{
 					return true;
+				}
+				
 				//TODO: Oh snap! No reusable error messages for this!
 				//Errors(CompilerErrorFactory.ConflictWithInheritedMember());
 			}
@@ -275,61 +312,52 @@ namespace Boo.Lang.Compiler.Steps
 			return false;
 		}
 
-		private IProperty GetPropertyEntity(TypeMember member)
-		{
-			return (IProperty)GetEntity(member);
-		}
-		
-		void ResolveClassAbstractProperty(ClassDefinition node,
+		void ResolveAbstractProperty(ClassDefinition node,
 			TypeReference baseTypeRef,
-			IProperty entity)
+			IProperty baseProperty)
 		{			
-			bool resolved = false;
-
-			foreach (TypeMember member in node.Members)
+			foreach (Property p in GetAbstractPropertyImplementationCandidates(node, baseProperty))
 			{
-				if (entity.Name != member.Name
-					|| NodeType.Property != member.NodeType
-					|| !IsCorrectExplicitMemberImplOrNoExplicitMemberAtAll(member, entity)
-					|| !TypeSystemServices.CheckOverrideSignature(entity.GetParameters(), GetPropertyEntity(member).GetParameters()))
+				if (!TypeSystemServices.CheckOverrideSignature(GetEntity(p).GetParameters(), baseProperty.GetParameters()))
+				{
 					continue;
+				}
 
-				Property p = (Property) member;
-				ProcessPropertyAccessor(p, p.Getter, entity.GetGetMethod());
-				ProcessPropertyAccessor(p, p.Setter, entity.GetSetMethod());
+				ProcessPropertyAccessor(p, p.Getter, baseProperty.GetGetMethod());
+				ProcessPropertyAccessor(p, p.Setter, baseProperty.GetSetMethod());
 				if (null == p.Type)
 				{
-					p.Type = CodeBuilder.CreateTypeReference(entity.Type);
+					p.Type = CodeBuilder.CreateTypeReference(baseProperty.Type);
 				}
 				else
 				{
-					if (entity.Type != p.Type.Entity)
-						Error(CompilerErrorFactory.ConflictWithInheritedMember(p, p.FullName, entity.FullName));
+					if (baseProperty.Type != p.Type.Entity)
+						Error(CompilerErrorFactory.ConflictWithInheritedMember(p, p.FullName, baseProperty.FullName));
 				}
-				resolved = true;
-			}
 
-			if (resolved)
-				return;
+				AssertValidInterfaceImplementation(p, baseProperty);
+
+				//fully-implemented?
+				if (!HasGetter(baseProperty) || (HasGetter(baseProperty) && null != p.Getter))
+					if (!HasSetter(baseProperty) || (HasSetter(baseProperty) && null != p.Setter))
+						return;
+			}
 
 			foreach(SimpleTypeReference parent in node.BaseTypes)
 			{
 				if(_classDefinitionList.Contains(parent.Name))
 				{
 					depth++;
-					ResolveClassAbstractProperty(_classDefinitionList[parent.Name] as ClassDefinition, baseTypeRef, entity);
+					ResolveAbstractProperty(_classDefinitionList[parent.Name] as ClassDefinition, baseTypeRef, baseProperty);
 					depth--;
 				}
 			}
 
-			if(CheckInheritsInterfaceImplementation(node, entity))
+			if(CheckInheritsInterfaceImplementation(node, baseProperty))
 				return;
 
 			if(depth == 0)
-			{
-				node.Members.Add(CreateAbstractProperty(baseTypeRef, entity));
-				AbstractMemberNotImplemented(node, baseTypeRef, entity);
-			}
+				AbstractMemberNotImplemented(node, baseTypeRef, baseProperty);
 		}
 
 		private static void ProcessPropertyAccessor(Property p, Method accessor, IMethod method)
@@ -346,35 +374,11 @@ namespace Boo.Lang.Compiler.Steps
 			}
 		}
 
-		Property CreateAbstractProperty(TypeReference reference, IProperty property)
+		void ResolveAbstractEvent(ClassDefinition node, TypeReference baseTypeRef, IEvent entity)
 		{
-			Debug.Assert(0 == property.GetParameters().Length);
-			Property p = CodeBuilder.CreateProperty(property.Name, property.Type);
-			p.Modifiers |= TypeMemberModifiers.Abstract;
-			
-			IMethod getter = property.GetGetMethod();
-			if (getter != null)
+			Event ev = node.Members[entity.Name] as Event;
+			if (ev != null)
 			{
-				p.Getter = CodeBuilder.CreateAbstractMethod(reference.LexicalInfo, getter);
-			}
-			
-			IMethod setter = property.GetSetMethod();
-			if (setter != null)
-			{
-				p.Setter = CodeBuilder.CreateAbstractMethod(reference.LexicalInfo, setter);
-			}
-			return p;
-		}
-		
-		void ResolveAbstractEvent(ClassDefinition node,
-			TypeReference baseTypeRef,
-			IEvent entity)
-		{			
-			TypeMember member = node.Members[entity.Name];
-			if (null != member)
-			{
-				Event ev = (Event)member;
-
 				Method add = ev.Add;
 				if (add != null)
 				{
@@ -393,6 +397,7 @@ namespace Boo.Lang.Compiler.Steps
 					raise.Modifiers |= TypeMemberModifiers.Final | TypeMemberModifiers.Virtual;
 				}
 
+				AssertValidInterfaceImplementation(ev, entity);
 				_context.TraceInfo("{0}: Event {1} implements {2}", ev.LexicalInfo, ev, entity);
 				return;
 			}
@@ -411,76 +416,127 @@ namespace Boo.Lang.Compiler.Steps
 			}
 			if(depth == 0)
 			{
-				node.Members.Add(CodeBuilder.CreateAbstractEvent(baseTypeRef.LexicalInfo, entity));
+				TypeMember conflicting;
+				if (null == ev && null != (conflicting = node.Members[entity.Name]))
+				{
+					//we've got a non-resolved conflicting member
+					Error(CompilerErrorFactory.ConflictWithInheritedMember(conflicting, conflicting.FullName, entity.FullName));
+					return;
+				}
+				AddStub(node, CodeBuilder.CreateAbstractEvent(baseTypeRef.LexicalInfo, entity));
 				AbstractMemberNotImplemented(node, baseTypeRef, entity);
 			}
 		}
 
 		void ResolveAbstractMethod(ClassDefinition node,
 			TypeReference baseTypeRef,
-			IMethod entity)
+			IMethod baseMethod)
 		{
-			if (entity.IsSpecialName)
+			if (baseMethod.IsSpecialName)
 				return;
 
-			bool resolved = false;
-
-			foreach (TypeMember member in node.Members)
+			foreach (Method method in GetAbstractMethodImplementationCandidates(node, baseMethod))
 			{
-				if (entity.Name == member.Name
-					&& NodeType.Method == member.NodeType
-					&& IsCorrectExplicitMemberImplOrNoExplicitMemberAtAll(member, entity))
+				IMethod methodEntity = GetEntity(method);
+
+				if (!TypeSystemServices.CheckOverrideSignature(methodEntity, baseMethod))
 				{
-					Method method = (Method)member;
-
-					if (TypeSystemServices.CheckOverrideSignature(GetEntity(method), entity))
-					{
-						if (IsUnknown(method.ReturnType))
-						{
-							method.ReturnType = CodeBuilder.CreateTypeReference(entity.ReturnType);
-						}
-						else if (GenericsServices.IsGenericParameter(method.ReturnType.Entity)
-									== GenericsServices.IsGenericParameter(entity.ReturnType))
-						{
-							if (!entity.ReturnType.Equals(method.ReturnType.Entity))
-								Error(CompilerErrorFactory.ConflictWithInheritedMember(method, method.FullName, entity.FullName));
-						}
-
-						if (null != method.ExplicitInfo)
-							method.ExplicitInfo.Entity = entity;
-
-						if (!method.IsOverride && !method.IsVirtual)
-							method.Modifiers |= TypeMemberModifiers.Virtual;
-
-						_context.TraceInfo("{0}: Method {1} implements {2}", method.LexicalInfo, method, entity);
-						resolved = true;
-					}
+					continue;
 				}
+
+				CallableSignature baseSignature = TypeSystemServices.GetOverriddenSignature(baseMethod, methodEntity);
+				if (IsUnknown(methodEntity.ReturnType))
+				{
+					method.ReturnType = CodeBuilder.CreateTypeReference(baseSignature.ReturnType);
+				}
+				else if (baseSignature.ReturnType != methodEntity.ReturnType)
+				{
+					Error(CompilerErrorFactory.ConflictWithInheritedMember(method, method.FullName, baseMethod.FullName));
+				}
+
+				if (null != method.ExplicitInfo)
+					method.ExplicitInfo.Entity = baseMethod;
+
+				if (!method.IsOverride && !method.IsVirtual)
+					method.Modifiers |= TypeMemberModifiers.Virtual;
+
+				AssertValidInterfaceImplementation(method, baseMethod);
+				_context.TraceInfo("{0}: Method {1} implements {2}", method.LexicalInfo, method, baseMethod);
+				return;
 			}
 
-			if (resolved)
-				return;
-
+			// FIXME: this will fail with InvalidCastException on a base type that's a GenericTypeReference!
 			foreach(SimpleTypeReference parent in node.BaseTypes)
 			{
 				if(_classDefinitionList.Contains(parent.Name))
 				{
 					depth++;
-					ResolveAbstractMethod(_classDefinitionList[parent.Name] as ClassDefinition, baseTypeRef, entity);
+					ResolveAbstractMethod(_classDefinitionList[parent.Name] as ClassDefinition, baseTypeRef, baseMethod);
 					depth--;
 				}
 			}
 
-			if(CheckInheritsInterfaceImplementation(node, entity))
+			if(CheckInheritsInterfaceImplementation(node, baseMethod))
 				return;
 
 			if(depth == 0)
 			{			
-				if (!AbstractMemberNotImplemented(node, baseTypeRef, entity))
+				if (!AbstractMemberNotImplemented(node, baseTypeRef, baseMethod))
 				{
 					//BEHAVIOR < 0.7.7: no stub, mark class as abstract
-					node.Members.Add(CodeBuilder.CreateAbstractMethod(baseTypeRef.LexicalInfo, entity));
-				}				
+					AddStub(node, CodeBuilder.CreateAbstractMethod(baseTypeRef.LexicalInfo, baseMethod));
+				}
+			}
+		}
+
+		private bool IsUnknown(IType type)
+		{
+			return TypeSystem.TypeSystemServices.IsUnknown(type);
+		}
+
+		private IEnumerable<Method> GetAbstractMethodImplementationCandidates(TypeDefinition node, IMethod baseMethod)
+		{
+			return GetAbstractMemberImplementationCandidates<Method, IMethod>(node, baseMethod);
+		}
+
+		private IEnumerable<Property> GetAbstractPropertyImplementationCandidates(TypeDefinition node, IProperty baseProperty)
+		{
+			return GetAbstractMemberImplementationCandidates<Property, IProperty>(node, baseProperty);
+		}
+
+		private IEnumerable<TMember> GetAbstractMemberImplementationCandidates<TMember, TEntity>(
+			TypeDefinition node, TEntity baseEntity)
+			where TEntity : IEntityWithParameters, IMember
+			where TMember : TypeMember, IExplicitMember		
+		{
+			List<TMember> candidates = new List<TMember>();
+			foreach (TypeMember m in node.Members)
+			{
+				TMember member = m as TMember;
+				if (member != null &&
+					member.Name == baseEntity.Name &&
+					IsCorrectExplicitMemberImplOrNoExplicitMemberAtAll(member, baseEntity))
+				{
+					candidates.Add(member);
+				}
+			}
+
+			// BOO-1031: Move explicitly implemented candidates to top of list so that
+			// they're used for resolution before non-explicit ones, if possible.
+			// HACK: using IComparer<T> instead of Comparison<T> to workaround
+			//       mono bug #399214.
+			candidates.Sort(new ExplicitMembersFirstComparer<TMember>());
+			return candidates;
+		}
+
+		private sealed class ExplicitMembersFirstComparer<T> : IComparer<T>
+			where T : IExplicitMember
+		{
+			public int Compare(T lhs, T rhs)
+			{
+				if (lhs.ExplicitInfo != null && rhs.ExplicitInfo == null) return -1;
+				if (lhs.ExplicitInfo == null && rhs.ExplicitInfo != null) return 1;
+				return 0;
 			}
 		}
 
@@ -510,13 +566,14 @@ namespace Boo.Lang.Compiler.Steps
 				//BEHAVIOR >= 0.7.7:	(see BOO-789 for details)
 				//create a stub for this not implemented member
 				//it will raise a NotImplementedException if called at runtime
-				TypeMember m = CodeBuilder.CreateStub(member);
-				CompilerWarning warning = null;				
+				TypeMember m = CodeBuilder.CreateStub(node, member);
+				CompilerWarning warning = null;
 				if (null != m)
 				{
 					warning = CompilerWarningFactory.AbstractMemberNotImplementedStubCreated(baseTypeRef,
 										node.FullName, GetAbstractMemberSignature(member));
-					node.Members.Add(m);
+					if (m.NodeType != NodeType.Property || null == node.Members[m.Name])
+						AddStub(node, m);
 				}
 				else
 				{
@@ -582,7 +639,7 @@ namespace Boo.Lang.Compiler.Steps
 						if (IsAbstractAccessor(property.GetGetMethod()) ||
 							IsAbstractAccessor(property.GetSetMethod()))
 						{
-							ResolveClassAbstractProperty(node, baseTypeRef, property);
+							ResolveAbstractProperty(node, baseTypeRef, property);
 						}
 						break;
 					}
@@ -624,7 +681,7 @@ namespace Boo.Lang.Compiler.Steps
 				
 				case EntityType.Property:
 				{
-					ResolveClassAbstractProperty(node, baseTypeRef, (IProperty)member);
+					ResolveAbstractProperty(node, baseTypeRef, (IProperty)member);
 					break;
 				}
 
@@ -649,6 +706,24 @@ namespace Boo.Lang.Compiler.Steps
 				node.Modifiers |= TypeMemberModifiers.Abstract;
 			}
 		}
+
+		void AddStub(TypeDefinition node, TypeMember stub)
+		{
+			node.Members.Add(stub);
+		}
+
+		void AssertValidInterfaceImplementation(TypeMember node, IMember baseMember)
+		{
+			if (!baseMember.DeclaringType.IsInterface)
+				return;
+
+			IExplicitMember explicitNode = node as IExplicitMember;
+			if (null != explicitNode && null != explicitNode.ExplicitInfo)
+				return; //node is an explicit interface impl
+
+			if (node.Visibility != TypeMemberModifiers.Public)
+				Errors.Add(CompilerErrorFactory.InterfaceImplementationMustBePublicOrExplicit(node, baseMember));
+		}
 	}
 }
 
