@@ -26,12 +26,15 @@
 // THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #endregion
 
+using System.Collections.Generic;
+using Boo.Lang.Compiler.TypeSystem.Services;
+
 namespace Boo.Lang.Compiler.TypeSystem
 {	
 	/// <summary>
 	/// A namespace.
 	/// </summary>
-	public interface INamespace
+	public interface INamespace : IEntity
 	{					
 		/// <summary>
 		/// The parent namespace.
@@ -44,47 +47,91 @@ namespace Boo.Lang.Compiler.TypeSystem
 		/// <summary>
 		/// Resolves the name passed as argument to the appropriate elements
 		/// in the namespace, all elements with the specified name must be
-		/// added to the targetList.
+		/// added to the resultingSet.
 		/// </summary>
-		/// <param name="targetList">list where to put the found elements</param>
+		/// <param name="resultingSet">list where to put the found elements</param>
 		/// <param name="name">name of the desired elements</param>
-		/// <param name="filter">element filter</param>
+		/// <param name="typesToConsider">element filter</param>
 		/// <returns>
-		/// true if at least one element was added to the targetList, false
+		/// true if at least one element was added to the resultingSet, false
 		/// otherwise.
 		/// </returns>
-		bool Resolve(List targetList, string name, EntityType filter);
+		bool Resolve(ICollection<IEntity> resultingSet, string name, EntityType typesToConsider);
 		
 		/// <summary>
 		/// Returns all members of this namespace.
 		/// </summary>
-		IEntity[] GetMembers();
+		IEnumerable<IEntity> GetMembers();
+	}
+
+	public abstract class AbstractNamespace : INamespace
+	{
+		#region Implementation of IEntity
+
+		public virtual string Name
+		{
+			get { return string.Empty; }
+		}
+
+		public virtual string FullName
+		{
+			get
+			{
+				INamespace parent = ParentNamespace;
+				if (parent == null)
+					return Name;
+
+				string parentName = parent.FullName;
+				return string.IsNullOrEmpty(parentName)
+				       	? Name
+				       	: parentName + "." + Name;
+			}
+		}
+
+		public virtual EntityType EntityType
+		{
+			get { return EntityType.Namespace; }
+		}
+
+		#endregion
+
+		#region Implementation of INamespace
+
+		public virtual INamespace ParentNamespace
+		{
+			get { return null; }
+		}
+
+		public virtual bool Resolve(ICollection<IEntity> resultingSet, string name, EntityType typesToConsider)
+		{
+			return My<NameResolutionService>.Instance.Resolve(name, GetMembers(), typesToConsider, resultingSet);
+		}
+
+		public abstract IEnumerable<IEntity> GetMembers();
+
+		#endregion
+
+		public override string ToString()
+		{
+			return FullName;
+		}
 	}
 	
-	public class NullNamespace : INamespace
+	public class NullNamespace : AbstractNamespace
 	{
 		public static readonly INamespace Default = new NullNamespace();
-		
 		public static readonly IEntity[] EmptyEntityArray = new IEntity[0];
 		
 		private NullNamespace()
 		{
 		}
-		
-		public INamespace ParentNamespace
-		{
-			get
-			{
-				return null;
-			}
-		}
-		
-		public bool Resolve(List targetList, string name, EntityType flags)
+
+		public override bool Resolve(ICollection<IEntity> resultingSet, string name, EntityType flags)
 		{
 			return false;
 		}
 		
-		public IEntity[] GetMembers()
+		public override IEnumerable<IEntity> GetMembers()
 		{
 			return NullNamespace.EmptyEntityArray;
 		}
