@@ -29,56 +29,33 @@
 
 namespace Boo.Lang.Compiler.TypeSystem
 {
-	public interface IType : ITypedEntity, INamespace
+	public interface IType : ITypedEntity, INamespace, IEntityWithAttributes
 	{	
-		bool IsClass
-		{
-			get;
-		}
+		bool IsClass { get; }
 		
-		bool IsAbstract
-		{
-			get;
-		}
+		bool IsAbstract { get; }
 		
-		bool IsInterface
-		{
-			get;
-		}
+		bool IsInterface { get; }
 		
-		bool IsEnum
-		{
-			get;
-		}
+		bool IsEnum { get; }
 		
-		bool IsByRef
-		{
-			get;
-		}
+		bool IsByRef { get; }
 		
-		bool IsValueType
-		{
-			get;
-		}
+		bool IsValueType { get; }
 
-		bool IsFinal
-		{
-			get;
-		}
+		bool IsFinal { get; }
 		
-		bool IsArray
-		{
-			get;
-		}
+		bool IsArray { get; }
+
+		bool IsPointer { get; }
+
+		IEntity DeclaringEntity { get; }
 		
 		int GetTypeDepth();
 		
 		IType GetElementType();
 		
-		IType BaseType
-		{
-			get;
-		}
+		IType BaseType { get; }
 		
 		IEntity GetDefaultMember();
 		
@@ -93,5 +70,10 @@ namespace Boo.Lang.Compiler.TypeSystem
 		IGenericTypeInfo GenericInfo { get; }
 		
 		IConstructedTypeInfo ConstructedInfo { get; }
+
+		IArrayType MakeArrayType(int rank);
+
+		IType MakePointerType();
 	}
 }
+
