@@ -132,30 +132,30 @@ using Boo.Lang.Parser.Util;
 		public const int VIRTUAL = 68;
 		public const int WHILE = 69;
 		public const int YIELD = 70;
-		public const int TRIPLE_QUOTED_STRING = 71;
-		public const int EOS = 72;
-		public const int DOUBLE_QUOTED_STRING = 73;
-		public const int SINGLE_QUOTED_STRING = 74;
-		public const int ID = 75;
+		public const int ID = 71;
+		public const int TRIPLE_QUOTED_STRING = 72;
+		public const int EOS = 73;
+		public const int DOUBLE_QUOTED_STRING = 74;
+		public const int SINGLE_QUOTED_STRING = 75;
 		public const int LBRACK = 76;
 		public const int RBRACK = 77;
 		public const int LPAREN = 78;
 		public const int RPAREN = 79;
 		public const int ASSIGN = 80;
-		public const int SUBTRACT = 81;
-		public const int COMMA = 82;
-		public const int ASSEMBLY_ATTRIBUTE_BEGIN = 83;
-		public const int SPLICE_BEGIN = 84;
-		public const int DOT = 85;
-		public const int COLON = 86;
-		public const int MULTIPLY = 87;
-		public const int NULLABLE_SUFFIX = 88;
-		public const int BITWISE_OR = 89;
-		public const int LBRACE = 90;
-		public const int RBRACE = 91;
-		public const int QQ_BEGIN = 92;
-		public const int QQ_END = 93;
-		public const int INPLACE_BITWISE_OR = 94;
+		public const int COMMA = 81;
+		public const int ASSEMBLY_ATTRIBUTE_BEGIN = 82;
+		public const int SPLICE_BEGIN = 83;
+		public const int DOT = 84;
+		public const int COLON = 85;
+		public const int MULTIPLY = 86;
+		public const int NULLABLE_SUFFIX = 87;
+		public const int BITWISE_OR = 88;
+		public const int LBRACE = 89;
+		public const int RBRACE = 90;
+		public const int QQ_BEGIN = 91;
+		public const int QQ_END = 92;
+		public const int INPLACE_BITWISE_OR = 93;
+		public const int INPLACE_EXCLUSIVE_OR = 94;
 		public const int INPLACE_BITWISE_AND = 95;
 		public const int INPLACE_SHIFT_LEFT = 96;
 		public const int INPLACE_SHIFT_RIGHT = 97;
@@ -163,41 +163,43 @@ using Boo.Lang.Parser.Util;
 		public const int GREATER_THAN = 99;
 		public const int LESS_THAN = 100;
 		public const int ADD = 101;
-		public const int EXCLUSIVE_OR = 102;
-		public const int DIVISION = 103;
-		public const int MODULUS = 104;
-		public const int BITWISE_AND = 105;
-		public const int SHIFT_LEFT = 106;
-		public const int SHIFT_RIGHT = 107;
-		public const int EXPONENTIATION = 108;
-		public const int INCREMENT = 109;
-		public const int DECREMENT = 110;
-		public const int ONES_COMPLEMENT = 111;
-		public const int INT = 112;
-		public const int LONG = 113;
-		public const int RE_LITERAL = 114;
-		public const int DOUBLE = 115;
-		public const int FLOAT = 116;
-		public const int TIMESPAN = 117;
-		public const int LINE_CONTINUATION = 118;
-		public const int SL_COMMENT = 119;
-		public const int ML_COMMENT = 120;
-		public const int WS = 121;
-		public const int X_RE_LITERAL = 122;
-		public const int NEWLINE = 123;
-		public const int ESCAPED_EXPRESSION = 124;
-		public const int DQS_ESC = 125;
-		public const int SQS_ESC = 126;
-		public const int SESC = 127;
-		public const int RE_CHAR = 128;
-		public const int X_RE_CHAR = 129;
-		public const int RE_ESC = 130;
-		public const int DIGIT_GROUP = 131;
-		public const int REVERSE_DIGIT_GROUP = 132;
-		public const int ID_PREFIX = 133;
-		public const int ID_LETTER = 134;
-		public const int DIGIT = 135;
-		public const int HEXDIGIT = 136;
+		public const int SUBTRACT = 102;
+		public const int EXCLUSIVE_OR = 103;
+		public const int DIVISION = 104;
+		public const int MODULUS = 105;
+		public const int BITWISE_AND = 106;
+		public const int SHIFT_LEFT = 107;
+		public const int SHIFT_RIGHT = 108;
+		public const int EXPONENTIATION = 109;
+		public const int INCREMENT = 110;
+		public const int DECREMENT = 111;
+		public const int ONES_COMPLEMENT = 112;
+		public const int INT = 113;
+		public const int LONG = 114;
+		public const int RE_LITERAL = 115;
+		public const int DOUBLE = 116;
+		public const int FLOAT = 117;
+		public const int TIMESPAN = 118;
+		public const int LINE_CONTINUATION = 119;
+		public const int SL_COMMENT = 120;
+		public const int ML_COMMENT = 121;
+		public const int WS = 122;
+		public const int X_RE_LITERAL = 123;
+		public const int NEWLINE = 124;
+		public const int ESCAPED_EXPRESSION = 125;
+		public const int DQS_ESC = 126;
+		public const int SQS_ESC = 127;
+		public const int SESC = 128;
+		public const int RE_CHAR = 129;
+		public const int X_RE_CHAR = 130;
+		public const int RE_OPTIONS = 131;
+		public const int RE_ESC = 132;
+		public const int DIGIT_GROUP = 133;
+		public const int REVERSE_DIGIT_GROUP = 134;
+		public const int ID_PREFIX = 135;
+		public const int ID_LETTER = 136;
+		public const int DIGIT = 137;
+		public const int HEXDIGIT = 138;
 		
 		
 	protected int _skipWhitespaceRegion = 0;
@@ -633,11 +635,11 @@ tryAgain:
 				}
 				else
 				{
-					goto _loop617_breakloop;
+					goto _loop627_breakloop;
 				}
 				
 			}
-_loop617_breakloop:			;
+_loop627_breakloop:			;
 		}    // ( ... )*
 		_ttype = testLiteralsTable(_ttype);
 		if (_createToken && (null == _token) && (_ttype != Token.SKIP))
@@ -809,7 +811,7 @@ _loop617_breakloop:			;
 			{
 				match("0x");
 				{ // ( ... )+
-					int _cnt622=0;
+					int _cnt632=0;
 					for (;;)
 					{
 						if ((tokenSet_2_.member(cached_LA1)))
@@ -818,12 +820,12 @@ _loop617_breakloop:			;
 						}
 						else
 						{
-							if (_cnt622 >= 1) { goto _loop622_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
+							if (_cnt632 >= 1) { goto _loop632_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
 						}
 						
-						_cnt622++;
+						_cnt632++;
 					}
-_loop622_breakloop:					;
+_loop632_breakloop:					;
 				}    // ( ... )+
 			}
 			{
@@ -1213,11 +1215,11 @@ _loop622_breakloop:					;
 				}
 				default:
 				{
-					goto _loop780_breakloop;
+					goto _loop795_breakloop;
 				}
 				 }
 			}
-_loop780_breakloop:			;
+_loop795_breakloop:			;
 		}    // ( ... )*
 		if (_createToken && (null == _token) && (_ttype != Token.SKIP))
 		{
@@ -1233,7 +1235,7 @@ _loop780_breakloop:			;
 		_ttype = REVERSE_DIGIT_GROUP;
 		
 		{ // ( ... )+
-			int _cnt784=0;
+			int _cnt799=0;
 			for (;;)
 			{
 				if (((cached_LA1 >= '0' && cached_LA1 <= '9')) && ((cached_LA2 >= '0' && cached_LA2 <= '9')) && ((LA(3) >= '0' && LA(3) <= '9')))
@@ -1259,12 +1261,12 @@ _loop780_breakloop:			;
 				}
 				else
 				{
-					if (_cnt784 >= 1) { goto _loop784_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
+					if (_cnt799 >= 1) { goto _loop799_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
 				}
 				
-				_cnt784++;
+				_cnt799++;
 			}
-_loop784_breakloop:			;
+_loop799_breakloop:			;
 		}    // ( ... )+
 		if (_createToken && (null == _token) && (_ttype != Token.SKIP))
 		{
@@ -1513,7 +1515,7 @@ _loop784_breakloop:			;
 				match('=');
 				if (0==inputState.guessing)
 				{
-					_ttype = ASSIGN;
+					_ttype = INPLACE_EXCLUSIVE_OR;
 				}
 			}
 			else {
@@ -1589,11 +1591,11 @@ _loop784_breakloop:			;
 			EnterSkipWhitespaceRegion();
 		}
 		{
-			bool synPredMatched666 = false;
+			bool synPredMatched676 = false;
 			if (((cached_LA1=='a')))
 			{
-				int _m666 = mark();
-				synPredMatched666 = true;
+				int _m676 = mark();
+				synPredMatched676 = true;
 				inputState.guessing++;
 				try {
 					{
@@ -1602,12 +1604,12 @@ _loop784_breakloop:			;
 				}
 				catch (RecognitionException)
 				{
-					synPredMatched666 = false;
+					synPredMatched676 = false;
 				}
-				rewind(_m666);
+				rewind(_m676);
 				inputState.guessing--;
 			}
-			if ( synPredMatched666 )
+			if ( synPredMatched676 )
 			{
 				match("assembly:");
 				if (0==inputState.guessing)
@@ -1815,6 +1817,19 @@ _loop784_breakloop:			;
 		_ttype = MODULUS;
 		
 		match('%');
+		{
+			if ((cached_LA1=='='))
+			{
+				match('=');
+				if (0==inputState.guessing)
+				{
+					_ttype = ASSIGN;
+				}
+			}
+			else {
+			}
+			
+		}
 		if (_createToken && (null == _token) && (_ttype != Token.SKIP))
 		{
 			_token = makeToken(_ttype);
@@ -1869,11 +1884,11 @@ _loop784_breakloop:			;
 		int _ttype; IToken _token=null; int _begin=text.Length;
 		_ttype = DIVISION;
 		
-		bool synPredMatched687 = false;
+		bool synPredMatched698 = false;
 		if (((cached_LA1=='/') && (cached_LA2=='*') && ((LA(3) >= '\u0003' && LA(3) <= '\ufffe'))))
 		{
-			int _m687 = mark();
-			synPredMatched687 = true;
+			int _m698 = mark();
+			synPredMatched698 = true;
 			inputState.guessing++;
 			try {
 				{
@@ -1882,12 +1897,12 @@ _loop784_breakloop:			;
 			}
 			catch (RecognitionException)
 			{
-				synPredMatched687 = false;
+				synPredMatched698 = false;
 			}
-			rewind(_m687);
+			rewind(_m698);
 			inputState.guessing--;
 		}
-		if ( synPredMatched687 )
+		if ( synPredMatched698 )
 		{
 			mML_COMMENT(false);
 			if (0==inputState.guessing)
@@ -1896,11 +1911,11 @@ _loop784_breakloop:			;
 			}
 		}
 		else {
-			bool synPredMatched689 = false;
+			bool synPredMatched700 = false;
 			if (((cached_LA1=='/') && (tokenSet_3_.member(cached_LA2)) && (tokenSet_4_.member(LA(3)))))
 			{
-				int _m689 = mark();
-				synPredMatched689 = true;
+				int _m700 = mark();
+				synPredMatched700 = true;
 				inputState.guessing++;
 				try {
 					{
@@ -1909,12 +1924,12 @@ _loop784_breakloop:			;
 				}
 				catch (RecognitionException)
 				{
-					synPredMatched689 = false;
+					synPredMatched700 = false;
 				}
-				rewind(_m689);
+				rewind(_m700);
 				inputState.guessing--;
 			}
-			if ( synPredMatched689 )
+			if ( synPredMatched700 )
 			{
 				mRE_LITERAL(false);
 				if (0==inputState.guessing)
@@ -1942,11 +1957,11 @@ _loop784_breakloop:			;
 									}
 									else
 									{
-										goto _loop694_breakloop;
+										goto _loop705_breakloop;
 									}
 									
 								}
-_loop694_breakloop:								;
+_loop705_breakloop:								;
 							}    // ( ... )*
 							if (0==inputState.guessing)
 							{
@@ -1999,11 +2014,11 @@ _loop694_breakloop:								;
 					match('*');
 				}
 				else {
-					bool synPredMatched735 = false;
+					bool synPredMatched746 = false;
 					if (((cached_LA1=='/') && (cached_LA2=='*') && ((LA(3) >= '\u0003' && LA(3) <= '\ufffe'))))
 					{
-						int _m735 = mark();
-						synPredMatched735 = true;
+						int _m746 = mark();
+						synPredMatched746 = true;
 						inputState.guessing++;
 						try {
 							{
@@ -2012,12 +2027,12 @@ _loop694_breakloop:								;
 						}
 						catch (RecognitionException)
 						{
-							synPredMatched735 = false;
+							synPredMatched746 = false;
 						}
-						rewind(_m735);
+						rewind(_m746);
 						inputState.guessing--;
 					}
-					if ( synPredMatched735 )
+					if ( synPredMatched746 )
 					{
 						mML_COMMENT(false);
 					}
@@ -2031,11 +2046,11 @@ _loop694_breakloop:								;
 					}
 					else
 					{
-						goto _loop737_breakloop;
+						goto _loop748_breakloop;
 					}
 					}
 				}
-_loop737_breakloop:				;
+_loop748_breakloop:				;
 			}    // ( ... )*
 			match("*/");
 			if (0==inputState.guessing)
@@ -2057,7 +2072,7 @@ _loop737_breakloop:				;
 		
 		match('/');
 		{ // ( ... )+
-			int _cnt766=0;
+			int _cnt777=0;
 			for (;;)
 			{
 				if ((tokenSet_3_.member(cached_LA1)))
@@ -2066,14 +2081,23 @@ _loop737_breakloop:				;
 				}
 				else
 				{
-					if (_cnt766 >= 1) { goto _loop766_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
+					if (_cnt777 >= 1) { goto _loop777_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
 				}
 				
-				_cnt766++;
+				_cnt777++;
 			}
-_loop766_breakloop:			;
+_loop777_breakloop:			;
 		}    // ( ... )+
 		match('/');
+		{
+			if ((tokenSet_1_.member(cached_LA1)))
+			{
+				mRE_OPTIONS(false);
+			}
+			else {
+			}
+			
+		}
 		if (_createToken && (null == _token) && (_ttype != Token.SKIP))
 		{
 			_token = makeToken(_ttype);
@@ -2290,12 +2314,12 @@ _loop766_breakloop:			;
 			for (;;)
 			{
 				// nongreedy exit test
-				if ((cached_LA1=='"') && (cached_LA2=='"') && (LA(3)=='"')) goto _loop715_breakloop;
-				bool synPredMatched711 = false;
+				if ((cached_LA1=='"') && (cached_LA2=='"') && (LA(3)=='"')) goto _loop726_breakloop;
+				bool synPredMatched722 = false;
 				if (((cached_LA1=='$') && (cached_LA2=='{') && ((LA(3) >= '\u0003' && LA(3) <= '\ufffe'))))
 				{
-					int _m711 = mark();
-					synPredMatched711 = true;
+					int _m722 = mark();
+					synPredMatched722 = true;
 					inputState.guessing++;
 					try {
 						{
@@ -2304,12 +2328,12 @@ _loop766_breakloop:			;
 					}
 					catch (RecognitionException)
 					{
-						synPredMatched711 = false;
+						synPredMatched722 = false;
 					}
-					rewind(_m711);
+					rewind(_m722);
 					inputState.guessing--;
 				}
-				if ( synPredMatched711 )
+				if ( synPredMatched722 )
 				{
 					if (0==inputState.guessing)
 					{
@@ -2321,11 +2345,11 @@ _loop766_breakloop:			;
 					mESCAPED_EXPRESSION(false);
 				}
 				else {
-					bool synPredMatched713 = false;
+					bool synPredMatched724 = false;
 					if (((cached_LA1=='\\') && (cached_LA2=='$') && ((LA(3) >= '\u0003' && LA(3) <= '\ufffe'))))
 					{
-						int _m713 = mark();
-						synPredMatched713 = true;
+						int _m724 = mark();
+						synPredMatched724 = true;
 						inputState.guessing++;
 						try {
 							{
@@ -2334,12 +2358,12 @@ _loop766_breakloop:			;
 						}
 						catch (RecognitionException)
 						{
-							synPredMatched713 = false;
+							synPredMatched724 = false;
 						}
-						rewind(_m713);
+						rewind(_m724);
 						inputState.guessing--;
 					}
-					if ( synPredMatched713 )
+					if ( synPredMatched724 )
 					{
 						_saveIndex = text.Length;
 						match('\\');
@@ -2356,11 +2380,11 @@ _loop766_breakloop:			;
 					}
 					else
 					{
-						goto _loop715_breakloop;
+						goto _loop726_breakloop;
 					}
 					}
 				}
-_loop715_breakloop:				;
+_loop726_breakloop:				;
 			}    // ( ... )*
 			_saveIndex = text.Length;
 			match("\"\"\"");
@@ -2428,11 +2452,11 @@ _loop715_breakloop:				;
 					{    // ( ... )*
 						for (;;)
 						{
-							bool synPredMatched721 = false;
+							bool synPredMatched732 = false;
 							if (((cached_LA1=='$') && (cached_LA2=='{') && (tokenSet_5_.member(LA(3)))))
 							{
-								int _m721 = mark();
-								synPredMatched721 = true;
+								int _m732 = mark();
+								synPredMatched732 = true;
 								inputState.guessing++;
 								try {
 									{
@@ -2441,12 +2465,12 @@ _loop715_breakloop:				;
 								}
 								catch (RecognitionException)
 								{
-									synPredMatched721 = false;
+									synPredMatched732 = false;
 								}
-								rewind(_m721);
+								rewind(_m732);
 								inputState.guessing--;
 							}
-							if ( synPredMatched721 )
+							if ( synPredMatched732 )
 							{
 								if (0==inputState.guessing)
 								{
@@ -2467,11 +2491,11 @@ _loop715_breakloop:				;
 							}
 							else
 							{
-								goto _loop723_breakloop;
+								goto _loop734_breakloop;
 							}
 							
 						}
-_loop723_breakloop:						;
+_loop734_breakloop:						;
 					}    // ( ... )*
 					_saveIndex = text.Length;
 					match('"');
@@ -2570,11 +2594,11 @@ _loop723_breakloop:						;
 				}
 				else
 				{
-					goto _loop727_breakloop;
+					goto _loop738_breakloop;
 				}
 				
 			}
-_loop727_breakloop:			;
+_loop738_breakloop:			;
 		}    // ( ... )*
 		_saveIndex = text.Length;
 		match('\'');
@@ -2642,11 +2666,11 @@ _loop727_breakloop:			;
 				}
 				else
 				{
-					goto _loop731_breakloop;
+					goto _loop742_breakloop;
 				}
 				
 			}
-_loop731_breakloop:			;
+_loop742_breakloop:			;
 		}    // ( ... )*
 		if (0==inputState.guessing)
 		{
@@ -2666,7 +2690,7 @@ _loop731_breakloop:			;
 		_ttype = WS;
 		
 		{ // ( ... )+
-			int _cnt740=0;
+			int _cnt751=0;
 			for (;;)
 			{
 				switch ( cached_LA1 )
@@ -2693,12 +2717,12 @@ _loop731_breakloop:			;
 				}
 				default:
 				{
-					if (_cnt740 >= 1) { goto _loop740_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
+					if (_cnt751 >= 1) { goto _loop751_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
 				}
 				break; }
-				_cnt740++;
+				_cnt751++;
 			}
-_loop740_breakloop:			;
+_loop751_breakloop:			;
 		}    // ( ... )+
 		if (0==inputState.guessing)
 		{
@@ -2742,7 +2766,7 @@ _loop740_breakloop:			;
 		text.Length = _saveIndex;
 		match('/');
 		{ // ( ... )+
-			int _cnt744=0;
+			int _cnt755=0;
 			for (;;)
 			{
 				if ((tokenSet_9_.member(cached_LA1)))
@@ -2751,12 +2775,12 @@ _loop740_breakloop:			;
 				}
 				else
 				{
-					if (_cnt744 >= 1) { goto _loop744_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
+					if (_cnt755 >= 1) { goto _loop755_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
 				}
 				
-				_cnt744++;
+				_cnt755++;
 			}
-_loop744_breakloop:			;
+_loop755_breakloop:			;
 		}    // ( ... )+
 		match('/');
 		if (0==inputState.guessing)
@@ -2987,6 +3011,36 @@ _loop744_breakloop:			;
 		returnToken_ = _token;
 	}
 	
+	protected void mRE_OPTIONS(bool _createToken) //throws RecognitionException, CharStreamException, TokenStreamException
+{
+		int _ttype; IToken _token=null; int _begin=text.Length;
+		_ttype = RE_OPTIONS;
+		
+		{ // ( ... )+
+			int _cnt784=0;
+			for (;;)
+			{
+				if ((tokenSet_1_.member(cached_LA1)))
+				{
+					mID_LETTER(false);
+				}
+				else
+				{
+					if (_cnt784 >= 1) { goto _loop784_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
+				}
+				
+				_cnt784++;
+			}
+_loop784_breakloop:			;
+		}    // ( ... )+
+		if (_createToken && (null == _token) && (_ttype != Token.SKIP))
+		{
+			_token = makeToken(_ttype);
+			_token.setText(text.ToString(_begin, text.Length-_begin));
+		}
+		returnToken_ = _token;
+	}
+	
 	protected void mRE_ESC(bool _createToken) //throws RecognitionException, CharStreamException, TokenStreamException
 {
 		int _ttype; IToken _token=null; int _begin=text.Length;
@@ -3119,7 +3173,7 @@ _loop744_breakloop:			;
 			case '8':  case '9':
 			{
 				{ // ( ... )+
-					int _cnt774=0;
+					int _cnt789=0;
 					for (;;)
 					{
 						if (((cached_LA1 >= '0' && cached_LA1 <= '9')) && (tokenSet_5_.member(cached_LA2)) && (true))
@@ -3128,12 +3182,12 @@ _loop744_breakloop:			;
 						}
 						else
 						{
-							if (_cnt774 >= 1) { goto _loop774_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
+							if (_cnt789 >= 1) { goto _loop789_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
 						}
 						
-						_cnt774++;
+						_cnt789++;
 					}
-_loop774_breakloop:					;
+_loop789_breakloop:					;
 				}    // ( ... )+
 				break;
 			}
