@@ -83,16 +83,16 @@ namespace BooC
 		
 		public int Run(string[] args)
 		{
-			int resultCode = -1;
-			
+			int resultCode = 127;
+
 			AppDomain.CurrentDomain.AssemblyResolve += new ResolveEventHandler(AssemblyResolve);
 			
 			CheckBooCompiler();
 			
 			try
 			{
-				DateTime start = DateTime.Now;
-				
+				Stopwatch setupTime = Stopwatch.StartNew();
+
 				_options = new CompilerParameters(false); //false means no stdlib loading yet
 				_options.GenerateInMemory = false;
 				
@@ -124,17 +124,17 @@ namespace BooC
 				LoadReferences();
 				ConfigurePipeline();
 				
-				if (_options.TraceSwitch.TraceInfo)
+				if (_options.TraceInfo)
 				{
 					compiler.Parameters.Pipeline.BeforeStep += new CompilerStepEventHandler(OnBeforeStep);
 					compiler.Parameters.Pipeline.AfterStep += new CompilerStepEventHandler(OnAfterStep);
 				}
-				
-				TimeSpan setupTime = DateTime.Now - start;
-				
-				start = DateTime.Now;
+
+				setupTime.Stop();
+
+				Stopwatch processingTime = Stopwatch.StartNew();
 				CompilerContext context = compiler.Run();
-				TimeSpan processingTime = DateTime.Now - start;
+				processingTime.Stop();
 
 				if (context.Warnings.Count > 0)
 				{
@@ -146,7 +146,7 @@ namespace BooC
 				{
 					foreach (CompilerError error in context.Errors)
 					{
-						Console.Error.WriteLine(error.ToString(_options.TraceSwitch.TraceInfo));
+						Console.Error.WriteLine(error.ToString(_options.TraceInfo));
 					}
 					Console.Error.WriteLine(Boo.Lang.ResourceManager.Format("BooC.Errors", context.Errors.Count));
 				}
@@ -155,14 +155,15 @@ namespace BooC
 					resultCode = 0;
 				}
 				
-				if (_options.TraceSwitch.TraceWarning)
+				if (_options.TraceWarning)
 				{
-					Console.Error.WriteLine(Boo.Lang.ResourceManager.Format("BooC.ProcessingTime", _options.Input.Count, processingTime.TotalMilliseconds, setupTime.TotalMilliseconds));
+					Console.Error.WriteLine(Boo.Lang.ResourceManager.Format("BooC.ProcessingTime", _options.Input.Count, processingTime.ElapsedMilliseconds, setupTime.ElapsedMilliseconds));
 				}
 			}
 			catch (Exception x)
 			{
-				object message = _options.TraceSwitch.TraceWarning ? (object)x : (object)x.Message;
+				object message = (_options.TraceWarning)
+									? (object)x : (object)x.Message;
 				Console.Error.WriteLine(Boo.Lang.ResourceManager.Format("BooC.FatalError", message));
 			}
 			return resultCode;
@@ -219,45 +220,50 @@ namespace BooC
 		
 		void DoLogo()
 		{
-			Console.Write("Boo Compiler version "+Builtins.BooVersion.ToString());
-			Console.WriteLine(" (CLR v"+Environment.Version.ToString()+")");
+			Console.WriteLine("Boo Compiler version {0} ({1})",
+				Builtins.BooVersion.ToString(), Boo.Lang.Runtime.RuntimeServices.RuntimeDisplayName);
 		}
 		
 		void Help ()
 		{
 			Console.WriteLine(
-					"Usage is: booc [options] file1 ...\n" +
+					"Usage: booc [options] file1 ...\n" +
 					"Options:\n" +
 					" -c:CULTURE           Sets the UI culture to be CULTURE\n" +
+					" -checked[+|-]        Turns on or off checked operations (default: +)\n" +
 					" -debug[+|-]          Generate debugging information (default: +)\n" +
 					" -define:S1[,Sn]      Defines symbols S1..Sn with optional values (=val) (-d:)\n" +
 					" -delaysign           Delays assembly signing\n" +
 					" -ducky               Turns on duck typing by default\n" +
-					" -checked[+|-]        Turns on or off checked operations (default: +)\n" +
 					" -embedres:FILE[,ID]  Embeds FILE with the optional ID\n"+
+					" -keycontainer:NAME   The key pair container used to strongname the assembly\n" +
+					" -keyfile:FILE        The strongname key file used to strongname the assembly\n" +
 					" -lib:DIRS            Adds the comma-separated DIRS to the assembly search path\n" +
 					" -noconfig            Does not load the standard configuration\n" +
-					" -nostdlib            Does not reference any of the default libraries\n" +
 					" -nologo              Does not display the compiler logo\n" +
-					" -p:PIPELINE          Sets the pipeline to PIPELINE\n" +
+					" -nostdlib            Does not reference any of the default libraries\n" +
+					" -nowarn[:W1,Wn]      Suppress all or a list of compiler warnings\n" +
 					" -o:FILE              Sets the output file name to FILE\n" +
-					" -keyfile:FILE        The strongname key file used to strongname the assembly\n" +
-					" -keycontainer:NAME   The key pair container used to strongname the assembly\n" +
-					" -reference:ASS       References the specified assembly (-r:ASS)\n" +
-					" -srcdir:DIR          Adds DIR as a directory where sources can be found\n" +
-					" -target:TYPE         Sets the target type (exe, library or winexe)\n" +
-					" -resource:FILE[,ID]  Embeds FILE as a resource\n" +
+					" -p:PIPELINE          Sets the pipeline to PIPELINE\n" +
 					" -pkg:P1[,Pn]         References packages P1..Pn (on supported platforms)\n" +
+					" -platform:ARCH       Specifies target platform (anycpu, x86, x64 or itanium)\n" +
+					" -reference:A1[,An]   References assemblies (-r:)\n" +
+					" -resource:FILE[,ID]  Embeds FILE as a resource\n" +
+					" -srcdir:DIR          Adds DIR as a directory where sources can be found\n" +
+					" -strict              Turns on strict mode.\n" +
+					" -target:TYPE         Sets the target type (exe, library or winexe) (-t:)\n" +
+					" -unsafe              Allows to compile unsafe code.\n" +
 					" -utf8                Source file(s) are in utf8 format\n" +
 					" -v, -vv, -vvv        Sets verbosity level from warnings to very detailed\n" +
+					" -warn:W1[,Wn]        Enables a list of optional warnings.\n" +
+					" -warnaserror[:W1,Wn] Treats all or a list of warnings as errors\n" +
 					" -wsa                 Enables white-space-agnostic build\n"
 					);
 		}
 
-		
 		void ParseOptions(string[] args)
 		{
-			bool noLogo = false;		
+			bool noLogo = false;
 			
 			ArrayList arglist = new ArrayList(args);
 			ExpandResponseFiles(ref arglist);
@@ -293,6 +299,26 @@ namespace BooC
 						{
 							_options.WhiteSpaceAgnostic = _whiteSpaceAgnostic = true;
 						}
+						else if (arg == "-warnaserror")
+						{
+							_options.WarnAsError = true;
+						}
+						else if (arg.StartsWith("-warnaserror:"))
+						{
+							string warnings = StripQuotes(arg.Substring(arg.IndexOf(":")+1));
+							foreach (string warning in warnings.Split(','))
+							{
+								_options.EnableWarningAsError(warning);
+							}
+						}
+						else if (arg.StartsWith("-warn:"))
+						{
+							string warnings = StripQuotes(arg.Substring(arg.IndexOf(":")+1));
+							foreach (string warning in warnings.Split(','))
+							{
+								_options.EnableWarning(warning);
+							}
+						}
 						else
 						{
 							InvalidOption(arg);
@@ -302,7 +328,8 @@ namespace BooC
 					
 					case 'v':
 					{
-						_options.TraceSwitch.Level = TraceLevel.Warning;
+						_options.EnableTraceSwitch();
+						_options.TraceLevel = TraceLevel.Warning;
 						Trace.Listeners.Add(new TextWriterTraceListener(Console.Error));
 						if (arg.Length > 2)
 						{
@@ -310,21 +337,21 @@ namespace BooC
 							{
 								case "vv":
 								{
-									_options.TraceSwitch.Level = TraceLevel.Info;
+									_options.TraceLevel = TraceLevel.Info;
 									MonitorAppDomain();
 									break;
 								}
 								
 								case "vvv":
 								{
-									_options.TraceSwitch.Level = TraceLevel.Verbose;
+									_options.TraceLevel = TraceLevel.Verbose;
 									break;
 								}
 							}
 						}
 						else
 						{
-							_options.TraceSwitch.Level = TraceLevel.Warning;
+							_options.TraceLevel = TraceLevel.Warning;
 						}
 						break;
 					}
@@ -351,8 +378,11 @@ namespace BooC
 						}
 						else
 						{
-							string assemblyName = StripQuotes(arg.Substring(arg.IndexOf(":")+1));
-							_references.Add(assemblyName);
+							string assemblies = StripQuotes(arg.Substring(arg.IndexOf(":")+1));
+							foreach (string assemblyName in assemblies.Split(','))
+							{
+								_references.Add(assemblyName);
+							}
 						}
 						break;
 					}
@@ -407,6 +437,18 @@ namespace BooC
 						{
 							_options.StdLib = false;
 						}
+						else if (arg == "-nowarn")
+						{
+							_options.NoWarn = true;
+						}
+						else if (arg.StartsWith("-nowarn:"))
+						{
+							string warnings = StripQuotes(arg.Substring(arg.IndexOf(":")+1));
+							foreach (string warning in warnings.Split(','))
+							{
+								_options.DisableWarning(warning);
+							}
+						}
 						else
 						{
 							InvalidOption(arg);
@@ -459,9 +501,35 @@ namespace BooC
 							string packages = StripQuotes(arg.Substring(arg.IndexOf(":")+1));
 							_packages.Add(packages);
 						}
-						else { 
+						else if (arg.StartsWith("-platform:"))
+						{
+							string arch = StripQuotes(arg.Substring(arg.IndexOf(":")+1)).ToLowerInvariant();
+							switch (arch)
+							{
+								case "anycpu":
+									break;
+								case "x86":
+									_options.Platform = "x86";
+									break;
+								case "x64":
+									_options.Platform = "x64";
+									break;
+								case "itanium":
+									_options.Platform = "itanium";
+									break;
+								default:
+									InvalidOption(arg, "Valid platform types are: `anycpu', `x86', `x64' or `itanium'.");
+									break;
+							}
+						}
+						else if (arg.StartsWith("-p:"))
+						{
 							_pipelineName = StripQuotes(arg.Substring(3));
 						}
+						else
+						{
+							InvalidOption(arg);
+						}
 						break;
 					}
 
@@ -483,13 +551,13 @@ namespace BooC
 							}
 							
 							default:
-							{							
+							{
 								string culture = arg.Substring(3);
 								Thread.CurrentThread.CurrentUICulture = CultureInfo.CreateSpecificCulture(culture);
 								break;
 							}
 						}
-						break;					
+						break;
 					}
 
 					case 's':
@@ -503,6 +571,16 @@ namespace BooC
 								break;
 							}
 
+							case "strict-":
+								break;
+
+							case "strict":
+							case "strict+":
+							{
+								_options.Strict = true;
+								break;
+							}
+
 							default:
 							{
 								InvalidOption(arg);
@@ -587,7 +665,7 @@ namespace BooC
 									}
 								}
 								else
-								{							
+								{
 									InvalidOption(arg);
 								}
 								break;
@@ -619,7 +697,16 @@ namespace BooC
 						}
 						break;
 					}
-					
+
+					case 'u':
+					{
+						if (arg == "-unsafe")
+							_options.Unsafe = true;
+						else
+							InvalidOption(arg);
+						break;
+					}
+
 					default:
 					{
 						if (arg == "--help")
@@ -643,7 +730,7 @@ namespace BooC
 
 		private bool IsMono
 		{
-			get { return Type.GetType("System.MonoType", false) != null;  }
+			get { return Type.GetType("Mono.Runtime") != null;  }
 		}
 
 		private void EmbedResource(string resourceFile)
@@ -714,7 +801,7 @@ namespace BooC
 			{
 				Console.WriteLine("********* {0} *********", args.Step);
 				
-				StringWriter writer = new StringWriter();				
+				StringWriter writer = new StringWriter();
 				args.Context.CompileUnit.Accept(new BooPrinterVisitor(writer, BooPrinterVisitor.PrintOptions.PrintLocals));
 				string code = writer.ToString();
 				if (code != _last)
@@ -819,20 +906,29 @@ namespace BooC
 			}
 			arglist = result;
 		}
-		
+
+		Stopwatch stepStopwatch;
+
 		void OnBeforeStep(object sender, CompilerStepEventArgs args)
 		{
 			args.Context.TraceEnter("Entering {0}", args.Step);
+			stepStopwatch = Stopwatch.StartNew();
 		}
 		
 		void OnAfterStep(object sender, CompilerStepEventArgs args)
 		{
-			args.Context.TraceLeave("Leaving {0}", args.Step);
+			stepStopwatch.Stop();
+			args.Context.TraceLeave("Leaving {0} ({1}ms)", args.Step, stepStopwatch.ElapsedMilliseconds);
 		}
-		
+
 		void InvalidOption(string arg)
 		{
-			Console.Error.WriteLine(Boo.Lang.ResourceManager.Format("BooC.InvalidOption", arg));
+			InvalidOption(arg, null);
+		}
+
+		void InvalidOption(string arg, string message)
+		{
+			Console.Error.WriteLine(Boo.Lang.ResourceManager.Format("BooC.InvalidOption", arg, message));
 		}
 
 		bool IsFlag(string arg)
@@ -856,7 +952,7 @@ namespace BooC
 		
 		void MonitorAppDomain()
 		{
-			if (_options.TraceSwitch.TraceInfo)
+			if (_options.TraceInfo)
 			{
 				AppDomain.CurrentDomain.AssemblyLoad += new AssemblyLoadEventHandler(OnAssemblyLoad);
 				foreach(Assembly a in AppDomain.CurrentDomain.GetAssemblies())
