@@ -27,6 +27,7 @@
 #endregion
 
 using System.Reflection;
+using System.Security;
 
 [assembly: AssemblyTitle("boo compiler")]
 [assembly: AssemblyDescription("")]
@@ -36,5 +37,6 @@ using System.Reflection;
 [assembly: AssemblyCopyright("(C) 2003-2007 Rodrigo Barreto de Oliveira")]
 [assembly: AssemblyTrademark("")]
 [assembly: AssemblyCulture("")]
-[assembly: AssemblyVersion("2.0.0.0")]
+[assembly: AssemblyVersion("2.0.9.2")]
 [assembly: AssemblyDelaySign(false)]
+[assembly: AllowPartiallyTrustedCallers]
