@@ -27,6 +27,7 @@
 #endregion
 
 using System.Reflection;
+using Boo.Lang.Compiler.TypeSystem.Reflection;
 
 namespace BooCompiler.Tests
 {
@@ -95,6 +96,7 @@ namespace BooCompiler.Tests
 			CopyAssembly(typeof(Boo.Lang.Extensions.MacroMacro).Assembly);
 			CopyAssembly(GetType().Assembly);
 			CopyAssembly(typeof(NUnit.Framework.Assert).Assembly);
+			CopyAssembly(System.Reflection.Assembly.Load("BooSupportingClasses"));
 #if !VISUAL_STUDIO
 			CopyAssembly(System.Reflection.Assembly.Load("BooModules"));
 #endif
@@ -116,7 +118,15 @@ namespace BooCompiler.Tests
 		
 		public void CopyAssembly(string location)
 		{
-			File.Copy(location, Path.Combine(Path.GetTempPath(), Path.GetFileName(location)), true);
+			string destFileName = Path.Combine(Path.GetTempPath(), Path.GetFileName(location));
+			if (File.Exists(destFileName) && !IsNewer(location, destFileName))
+				return;
+			File.Copy(location, destFileName, true);
+		}
+
+		private bool IsNewer(string fileName, string thanFileName)
+		{
+			return File.GetLastWriteTime(fileName) > File.GetLastWriteTime(thanFileName);
 		}
 
 		[TestFixtureTearDown]
@@ -130,7 +140,11 @@ namespace BooCompiler.Tests
 		{
 			System.Threading.Thread current = System.Threading.Thread.CurrentThread;
 
+			_parameters.OutputType = CompilerOutputType.Auto;
 			_parameters.Input.Clear();
+			_parameters.Strict = false;
+			_parameters.ResetWarnings();
+			_parameters.ResetWarningsAsErrors();
 
 			current.CurrentCulture = current.CurrentUICulture = System.Globalization.CultureInfo.InvariantCulture;
 		}
@@ -230,7 +244,7 @@ namespace BooCompiler.Tests
 				{
 					Assert.Fail(GetFirstInputName(context)
 								+ ": "
-								+ context.Errors.ToString(false)
+								+ context.Errors.ToString(true)
 								+ context.Warnings.ToString());				
 				}
 				return _output.ToString().Replace("\r\n", "\n");
