@@ -4,18 +4,14 @@ namespace BooCompiler.Tests
 	using NUnit.Framework;
 	using Boo.Lang.Compiler;
 	using Boo.Lang.Compiler.Steps;
+	using Boo.Lang.Compiler.Steps.MacroProcessing;
 
 	[TestFixture]
 	public class AttributesTestFixture : AbstractCompilerTestCase
 	{
 		override protected CompilerPipeline SetUpCompilerPipeline()
 		{
-			CompilerPipeline pipeline = new Boo.Lang.Compiler.Pipelines.Parse();
-			pipeline.Add(new InitializeTypeSystemServices());
-			pipeline.Add(new InitializeNameResolutionService());
-			pipeline.Add(new IntroduceGlobalNamespaces());
-			pipeline.Add(new BindNamespaces());
-			pipeline.Add(new BindAndApplyAttributes());
+			CompilerPipeline pipeline = new Boo.Lang.Compiler.Pipelines.ExpandMacros();
 			pipeline.Add(new PrintBoo());
 			return pipeline;
 		}
@@ -74,6 +70,12 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"property-5.boo");
 		}
 		
+		[Test]
+		public void property_6()
+		{
+			RunCompilerTestCase(@"property-6.boo");
+		}
+		
 		[Test]
 		public void required_1()
 		{
