@@ -26,6 +26,12 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"BCE0004-2.boo");
 		}
 		
+		[Test]
+		public void BCE0004_3()
+		{
+			RunCompilerTestCase(@"BCE0004-3.boo");
+		}
+		
 		[Test]
 		public void BCE0005_1()
 		{
@@ -44,6 +50,12 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"BCE0006-1.boo");
 		}
 		
+		[Test]
+		public void BCE0007_1()
+		{
+			RunCompilerTestCase(@"BCE0007-1.boo");
+		}
+		
 		[Test]
 		public void BCE0017_1()
 		{
@@ -62,6 +74,12 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"BCE0017-3.boo");
 		}
 		
+		[Test]
+		public void BCE0017_4()
+		{
+			RunCompilerTestCase(@"BCE0017-4.boo");
+		}
+		
 		[Test]
 		public void BCE0018_1()
 		{
@@ -134,6 +152,18 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"BCE0022-11.boo");
 		}
 		
+		[Test]
+		public void BCE0022_12()
+		{
+			RunCompilerTestCase(@"BCE0022-12.boo");
+		}
+		
+		[Test]
+		public void BCE0022_13()
+		{
+			RunCompilerTestCase(@"BCE0022-13.boo");
+		}
+		
 		[Test]
 		public void BCE0022_2()
 		{
@@ -236,12 +266,54 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"BCE0035-3.boo");
 		}
 		
+		[Test]
+		public void BCE0035_4()
+		{
+			RunCompilerTestCase(@"BCE0035-4.boo");
+		}
+		
+		[Test]
+		public void BCE0038_1()
+		{
+			RunCompilerTestCase(@"BCE0038-1.boo");
+		}
+		
+		[Test]
+		public void BCE0045_1()
+		{
+			RunCompilerTestCase(@"BCE0045-1.boo");
+		}
+		
+		[Test]
+		public void BCE0045_2()
+		{
+			RunCompilerTestCase(@"BCE0045-2.boo");
+		}
+		
+		[Test]
+		public void BCE0045_3()
+		{
+			RunCompilerTestCase(@"BCE0045-3.boo");
+		}
+		
+		[Test]
+		public void BCE0045_4()
+		{
+			RunCompilerTestCase(@"BCE0045-4.boo");
+		}
+		
 		[Test]
 		public void BCE0046_1()
 		{
 			RunCompilerTestCase(@"BCE0046-1.boo");
 		}
 		
+		[Test]
+		public void BCE0046_2()
+		{
+			RunCompilerTestCase(@"BCE0046-2.boo");
+		}
+		
 		[Test]
 		public void BCE0047_1()
 		{
@@ -314,6 +386,12 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"BCE0060-2.boo");
 		}
 		
+		[Test]
+		public void BCE0060_3()
+		{
+			RunCompilerTestCase(@"BCE0060-3.boo");
+		}
+		
 		[Test]
 		public void BCE0061_1()
 		{
@@ -350,6 +428,12 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"BCE0065-1.boo");
 		}
 		
+		[Test]
+		public void BCE0065_2()
+		{
+			RunCompilerTestCase(@"BCE0065-2.boo");
+		}
+		
 		[Test]
 		public void BCE0067_1()
 		{
@@ -596,6 +680,12 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"BCE0090-2.boo");
 		}
 		
+		[Test]
+		public void BCE0090_3()
+		{
+			RunCompilerTestCase(@"BCE0090-3.boo");
+		}
+		
 		[Test]
 		public void BCE0091_1()
 		{
@@ -794,6 +884,18 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"BCE0120-4.boo");
 		}
 		
+		[Test]
+		public void BCE0120_5()
+		{
+			RunCompilerTestCase(@"BCE0120-5.boo");
+		}
+		
+		[Test]
+		public void BCE0120_6()
+		{
+			RunCompilerTestCase(@"BCE0120-6.boo");
+		}
+		
 		[Test]
 		public void BCE0121_1()
 		{
@@ -812,6 +914,12 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"BCE0123-1.boo");
 		}
 		
+		[Test]
+		public void BCE0123_2()
+		{
+			RunCompilerTestCase(@"BCE0123-2.boo");
+		}
+		
 		[Test]
 		public void BCE0124_1()
 		{
@@ -860,6 +968,12 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"BCE0127-1.boo");
 		}
 		
+		[Test]
+		public void BCE0127_2()
+		{
+			RunCompilerTestCase(@"BCE0127-2.boo");
+		}
+		
 		[Test]
 		public void BCE0128_1()
 		{
@@ -1058,6 +1172,78 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"BCE0157-1.boo");
 		}
 		
+		[Test]
+		public void BCE0158_1()
+		{
+			RunCompilerTestCase(@"BCE0158-1.boo");
+		}
+		
+		[Test]
+		public void BCE0165_1()
+		{
+			RunCompilerTestCase(@"BCE0165-1.boo");
+		}
+		
+		[Test]
+		public void BCE0166_1()
+		{
+			RunCompilerTestCase(@"BCE0166-1.boo");
+		}
+		
+		[Test]
+		public void BCE0167_1()
+		{
+			RunCompilerTestCase(@"BCE0167-1.boo");
+		}
+		
+		[Test]
+		public void BCE0169_1()
+		{
+			RunCompilerTestCase(@"BCE0169-1.boo");
+		}
+		
+		[Test]
+		public void BCE0170_1()
+		{
+			RunCompilerTestCase(@"BCE0170-1.boo");
+		}
+		
+		[Test]
+		public void BCE0171_1()
+		{
+			RunCompilerTestCase(@"BCE0171-1.boo");
+		}
+		
+		[Test]
+		public void BCE0173_1()
+		{
+			RunCompilerTestCase(@"BCE0173-1.boo");
+		}
+		
+		[Test]
+		public void invalid_generic_extension_1()
+		{
+			RunCompilerTestCase(@"invalid-generic-extension-1.boo");
+		}
+		
+		[Test]
+		public void invalid_generic_extension_2()
+		{
+			RunCompilerTestCase(@"invalid-generic-extension-2.boo");
+		}
+		
+		[Test]
+		public void strict_1()
+		{
+			RunCompilerTestCase(@"strict-1.boo");
+		}
+		
+		[Test]
+		public void warnaserror_1()
+		{
+			RunCompilerTestCase(@"warnaserror-1.boo");
+		}
+		
 
 		override protected string GetRelativeTestCasesPath()
 		{
