@@ -132,6 +132,12 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"generators-9.boo");
 		}
 		
+		[Ignore(" BO0-759 - generic generator methods are not supported")][Test]
+		public void generic_generator_1()
+		{
+			RunCompilerTestCase(@"generic-generator-1.boo");
+		}
+		
 		[Test]
 		public void label_issue_1()
 		{
