@@ -18,6 +18,12 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"auto-lift-2.boo");
 		}
 		
+		[Test]
+		public void block_lift()
+		{
+			RunCompilerTestCase(@"block-lift.boo");
+		}
+		
 		[Test]
 		public void compile_1()
 		{
@@ -90,6 +96,12 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"meta-methods-6.boo");
 		}
 		
+		[Test]
+		public void meta_methods_can_return_null()
+		{
+			RunCompilerTestCase(@"meta-methods-can-return-null.boo");
+		}
+		
 		[Test]
 		public void name_splicing_1()
 		{
@@ -162,6 +174,12 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"quasi-quotation-3.boo");
 		}
 		
+		[Test]
+		public void quasi_quotation_4()
+		{
+			RunCompilerTestCase(@"quasi-quotation-4.boo");
+		}
+		
 		[Test]
 		public void splicing_1()
 		{
@@ -216,6 +234,12 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"splicing-9.boo");
 		}
 		
+		[Test]
+		public void typedef_splicing_1()
+		{
+			RunCompilerTestCase(@"typedef-splicing-1.boo");
+		}
+		
 		[Test]
 		public void typeref_splicing_1()
 		{
