@@ -25,6 +25,12 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"BOO-1005-2.boo");
 		}
 		
+		[Test]
+		public void BOO_1006_1()
+		{
+			RunCompilerTestCase(@"BOO-1006-1.boo");
+		}
+		
 		[Test]
 		public void BOO_1008_1()
 		{
@@ -61,24 +67,246 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"BOO-1016-2.boo");
 		}
 		
+		[Test]
+		public void BOO_1027_1()
+		{
+			RunCompilerTestCase(@"BOO-1027-1.boo");
+		}
+		
+		[Test]
+		public void BOO_1031_1()
+		{
+			RunCompilerTestCase(@"BOO-1031-1.boo");
+		}
+		
+		[Test]
+		public void BOO_1031_2()
+		{
+			RunCompilerTestCase(@"BOO-1031-2.boo");
+		}
+		
+		[Test]
+		public void BOO_1031_3()
+		{
+			RunCompilerTestCase(@"BOO-1031-3.boo");
+		}
+		
+		[Test]
+		public void BOO_1031_4()
+		{
+			RunCompilerTestCase(@"BOO-1031-4.boo");
+		}
+		
+		[Test]
+		public void boo_1032_1()
+		{
+			RunCompilerTestCase(@"boo-1032-1.boo");
+		}
+		
+		[Test]
+		public void boo_1032_2()
+		{
+			RunCompilerTestCase(@"boo-1032-2.boo");
+		}
+		
+		[Test]
+		public void BOO_1035_1()
+		{
+			RunCompilerTestCase(@"BOO-1035-1.boo");
+		}
+		
 		[Test]
 		public void BOO_104_1()
 		{
 			RunCompilerTestCase(@"BOO-104-1.boo");
 		}
 		
+		[Test]
+		public void BOO_1040_1()
+		{
+			RunCompilerTestCase(@"BOO-1040-1.boo");
+		}
+		
+		[Test]
+		public void BOO_1047()
+		{
+			RunCompilerTestCase(@"BOO-1047.boo");
+		}
+		
+		[Test]
+		public void boo_1051()
+		{
+			RunCompilerTestCase(@"boo-1051.boo");
+		}
+		
+		[Test]
+		public void BOO_1069_1()
+		{
+			RunCompilerTestCase(@"BOO-1069-1.boo");
+		}
+		
+		[Test]
+		public void BOO_1070_1()
+		{
+			RunCompilerTestCase(@"BOO-1070-1.boo");
+		}
+		
+		[Test]
+		public void BOO_1076()
+		{
+			RunCompilerTestCase(@"BOO-1076.boo");
+		}
+		
+		[Test]
+		public void BOO_1078_1()
+		{
+			RunCompilerTestCase(@"BOO-1078-1.boo");
+		}
+		
+		[Test]
+		public void BOO_1080_1()
+		{
+			RunCompilerTestCase(@"BOO-1080-1.boo");
+		}
+		
+		[Test]
+		public void BOO_1085()
+		{
+			RunCompilerTestCase(@"BOO-1085.boo");
+		}
+		
+		[Test]
+		public void BOO_1086()
+		{
+			RunCompilerTestCase(@"BOO-1086.boo");
+		}
+		
+		[Test]
+		public void BOO_1088_1()
+		{
+			RunCompilerTestCase(@"BOO-1088-1.boo");
+		}
+		
+		[Test]
+		public void BOO_1091_1()
+		{
+			RunCompilerTestCase(@"BOO-1091-1.boo");
+		}
+		
+		[Test]
+		public void BOO_1095_1()
+		{
+			RunCompilerTestCase(@"BOO-1095-1.boo");
+		}
+		
+		[Test]
+		public void BOO_1111_1()
+		{
+			RunCompilerTestCase(@"BOO-1111-1.boo");
+		}
+		
+		[Test]
+		public void BOO_1127_1()
+		{
+			RunCompilerTestCase(@"BOO-1127-1.boo");
+		}
+		
+		[Test]
+		public void BOO_1130_1()
+		{
+			RunCompilerTestCase(@"BOO-1130-1.boo");
+		}
+		
+		[Test]
+		public void BOO_1135_1()
+		{
+			RunCompilerTestCase(@"BOO-1135-1.boo");
+		}
+		
+		[Test]
+		public void BOO_1147_1()
+		{
+			RunCompilerTestCase(@"BOO-1147-1.boo");
+		}
+		
+		[Ignore(" Boo's SRE of generic base type constraint of constraint's declaring type does not work on MS.NET (2.0 at least)")][Test]
+		public void BOO_1147_2()
+		{
+			RunCompilerTestCase(@"BOO-1147-2.boo");
+		}
+		
+		[Test]
+		public void BOO_1154_1()
+		{
+			RunCompilerTestCase(@"BOO-1154-1.boo");
+		}
+		
+		[Test]
+		public void BOO_1160_1()
+		{
+			RunCompilerTestCase(@"BOO-1160-1.boo");
+		}
+		
+		[Test]
+		public void BOO_1162_1()
+		{
+			RunCompilerTestCase(@"BOO-1162-1.boo");
+		}
+		
+		[Test]
+		public void BOO_1171_1()
+		{
+			RunCompilerTestCase(@"BOO-1171-1.boo");
+		}
+		
+		[Test]
+		public void BOO_1176_1()
+		{
+			RunCompilerTestCase(@"BOO-1176-1.boo");
+		}
+		
+		[Test]
+		public void BOO_1177_1()
+		{
+			RunCompilerTestCase(@"BOO-1177-1.boo");
+		}
+		
+		[Test]
+		public void BOO_1206_1()
+		{
+			RunCompilerTestCase(@"BOO-1206-1.boo");
+		}
+		
 		[Test]
 		public void BOO_121_1()
 		{
 			RunCompilerTestCase(@"BOO-121-1.boo");
 		}
 		
+		[Test]
+		public void BOO_1210_1()
+		{
+			RunCompilerTestCase(@"BOO-1210-1.boo");
+		}
+		
+		[Test]
+		public void BOO_1217_1()
+		{
+			RunCompilerTestCase(@"BOO-1217-1.boo");
+		}
+		
 		[Test]
 		public void BOO_122_1()
 		{
 			RunCompilerTestCase(@"BOO-122-1.boo");
 		}
 		
+		[Test]
+		public void BOO_1220_1()
+		{
+			RunCompilerTestCase(@"BOO-1220-1.boo");
+		}
+		
 		[Test]
 		public void BOO_123_1()
 		{
@@ -817,6 +1045,12 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"BOO-799-1.boo");
 		}
 		
+		[Test]
+		public void BOO_806_1()
+		{
+			RunCompilerTestCase(@"BOO-806-1.boo");
+		}
+		
 		[Test]
 		public void BOO_809_1()
 		{
@@ -949,6 +1183,18 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"BOO-913-1.boo");
 		}
 		
+		[Test]
+		public void BOO_935_1()
+		{
+			RunCompilerTestCase(@"BOO-935-1.boo");
+		}
+		
+		[Test]
+		public void BOO_935_2()
+		{
+			RunCompilerTestCase(@"BOO-935-2.boo");
+		}
+		
 		[Test]
 		public void BOO_948_1()
 		{
@@ -967,6 +1213,12 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"BOO-949-2.boo");
 		}
 		
+		[Test]
+		public void BOO_952_1()
+		{
+			RunCompilerTestCase(@"BOO-952-1.boo");
+		}
+		
 		[Test]
 		public void BOO_955_1()
 		{
@@ -1069,6 +1321,12 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"BOO-999-1.boo");
 		}
 		
+		[Test]
+		public void duck_default_member_overload()
+		{
+			RunCompilerTestCase(@"duck-default-member-overload.boo");
+		}
+		
 		[Test]
 		public void for_re_Split()
 		{
@@ -1081,6 +1339,12 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"generators-1.boo");
 		}
 		
+		[Test]
+		public void override_inference()
+		{
+			RunCompilerTestCase(@"override-inference.boo");
+		}
+		
 
 		override protected string GetRelativeTestCasesPath()
 		{
