@@ -126,6 +126,12 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"classes-9.boo");
 		}
 		
+		[Test]
+		public void constructor_return_1()
+		{
+			RunCompilerTestCase(@"constructor-return-1.boo");
+		}
+		
 		[Test]
 		public void constructors_1()
 		{
@@ -180,6 +186,12 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"enums-16.boo");
 		}
 		
+		[Test]
+		public void enums_17()
+		{
+			RunCompilerTestCase(@"enums-17.boo");
+		}
+		
 		[Test]
 		public void enums_2()
 		{
@@ -426,6 +438,12 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"innerclasses-11.boo");
 		}
 		
+		[Test]
+		public void innerclasses_12()
+		{
+			RunCompilerTestCase(@"innerclasses-12.boo");
+		}
+		
 		[Test]
 		public void innerclasses_2()
 		{
@@ -606,6 +624,12 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"interfaces-9.boo");
 		}
 		
+		[Test]
+		public void internal_base_type_is_preferred()
+		{
+			RunCompilerTestCase(@"internal-base-type-is-preferred.boo");
+		}
+		
 		[Test]
 		public void methods_1()
 		{
@@ -1068,6 +1092,12 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"value-types-20.boo");
 		}
 		
+		[Test]
+		public void value_types_21()
+		{
+			RunCompilerTestCase(@"value-types-21.boo");
+		}
+		
 		[Test]
 		public void value_types_3()
 		{
